import json

from . import AWSObject, AWSProperty
from .compat import policytypes
from .constants import LOGS_ALLOWED_RETENTION_DAYS as RETENTION_DAYS
from .validators import integer_list_item, json_checker

policytypes = policytypes + (str,)


def validate_resource_policy(policy_document):
    """validate policy_document. Between 1 to 5120"""

    if not isinstance(policy_document, policytypes):
        raise ValueError("PolicyDocument must be a valid policy document")

    if isinstance(policy_document, str) and not json_checker(policy_document):
        raise ValueError("PolicyDocument must be a valid JSON formated string")

    if isinstance(policy_document, dict):
        policy_document_text = json.dumps(policy_document)
    elif isinstance(policy_document, str):
        policy_document_text = policy_document
    else:
        policy_document_text = policy_document.to_json()

    # NB: {} empty dict is 2 length
    if len(policy_document_text) < 3:
        raise ValueError("PolicyDocument must not be empty")

    if len(policy_document_text) > 5120:
        raise ValueError("PolicyDocument maximum length must not exceed 5120")

    return policy_document


class Destination(AWSObject):
    resource_type = "AWS::Logs::Destination"

    props = {
        "DestinationName": (str, True),
        "DestinationPolicy": (str, True),
        "RoleArn": (str, True),
        "TargetArn": (str, True),
    }


class LogGroup(AWSObject):
    resource_type = "AWS::Logs::LogGroup"

    props = {
        "KmsKeyId": (str, False),
        "LogGroupName": (str, False),
        "RetentionInDays": (integer_list_item(RETENTION_DAYS), False),
    }


class LogStream(AWSObject):
    resource_type = "AWS::Logs::LogStream"

    props = {"LogGroupName": (str, True), "LogStreamName": (str, False)}


class MetricTransformation(AWSProperty):
    props = {
        "DefaultValue": (float, False),
        "MetricName": (str, True),
        "MetricNamespace": (str, True),
        "MetricValue": (str, True),
    }


class MetricFilter(AWSObject):
    resource_type = "AWS::Logs::MetricFilter"

    props = {
        "FilterPattern": (str, True),
        "LogGroupName": (str, True),
        "MetricTransformations": ([MetricTransformation], True),
    }


class QueryDefinition(AWSObject):
    resource_type = "AWS::Logs::QueryDefinition"

    props = {
        "LogGroupNames": ([str], False),
        "Name": (str, True),
        "QueryString": (str, True),
    }


class ResourcePolicy(AWSObject):
    resource_type = "AWS::Logs::ResourcePolicy"

    props = {
        "PolicyDocument": (validate_resource_policy, True),
        "PolicyName": (str, True),
    }


class SubscriptionFilter(AWSObject):
    resource_type = "AWS::Logs::SubscriptionFilter"

    props = {
        "DestinationArn": (str, True),
        "FilterPattern": (str, True),
        "LogGroupName": (str, True),
        "RoleArn": (str, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, double


def resolver_kind_validator(x):
    valid_types = ["UNIT", "PIPELINE"]
    if x not in valid_types:
        raise ValueError("Kind must be one of: %s" % ", ".join(valid_types))
    return x


class ApiCache(AWSObject):
    resource_type = "AWS::AppSync::ApiCache"

    props = {
        "ApiCachingBehavior": (str, True),
        "ApiId": (str, True),
        "AtRestEncryptionEnabled": (boolean, False),
        "TransitEncryptionEnabled": (boolean, False),
        "Ttl": (double, True),
        "Type": (str, True),
    }


class ApiKey(AWSObject):
    resource_type = "AWS::AppSync::ApiKey"

    props = {
        "ApiId": (str, True),
        "ApiKeyId": (str, False),
        "Description": (str, False),
        "Expires": (double, False),
    }


class DeltaSyncConfig(AWSProperty):
    props = {
        "BaseTableTTL": (str, True),
        "DeltaSyncTableName": (str, True),
        "DeltaSyncTableTTL": (str, True),
    }


class DynamoDBConfig(AWSProperty):
    props = {
        "AwsRegion": (str, True),
        "DeltaSyncConfig": (DeltaSyncConfig, False),
        "TableName": (str, True),
        "UseCallerCredentials": (boolean, False),
        "Versioned": (boolean, False),
    }


class ElasticsearchConfig(AWSProperty):
    props = {
        "AwsRegion": (str, True),
        "Endpoint": (str, True),
    }


class AwsIamConfig(AWSProperty):
    props = {
        "SigningRegion": (str, False),
        "SigningServiceName": (str, False),
    }


class AuthorizationConfig(AWSProperty):
    props = {
        "AuthorizationType": (str, True),
        "AwsIamConfig": (AwsIamConfig, False),
    }


class HttpConfig(AWSProperty):
    props = {
        "AuthorizationConfig": (AuthorizationConfig, False),
        "Endpoint": (str, True),
    }


class LambdaConfig(AWSProperty):
    props = {
        "LambdaFunctionArn": (str, True),
    }


class RdsHttpEndpointConfig(AWSProperty):
    props = {
        "AwsRegion": (str, True),
        "AwsSecretStoreArn": (str, True),
        "DatabaseName": (str, False),
        "DbClusterIdentifier": (str, True),
        "Schema": (str, False),
    }


class RelationalDatabaseConfig(AWSProperty):
    props = {
        "RdsHttpEndpointConfig": (RdsHttpEndpointConfig, False),
        "RelationalDatasourceType": (str, True),
    }


class DataSource(AWSObject):
    resource_type = "AWS::AppSync::DataSource"

    props = {
        "ApiId": (str, True),
        "Description": (str, False),
        "DynamoDBConfig": (DynamoDBConfig, False),
        "ElasticsearchConfig": (ElasticsearchConfig, False),
        "HttpConfig": (HttpConfig, False),
        "LambdaConfig": (LambdaConfig, False),
        "Name": (str, True),
        "RelationalDatabaseConfig": (RelationalDatabaseConfig, False),
        "ServiceRoleArn": (str, False),
        "Type": (str, True),
    }


class FunctionConfiguration(AWSObject):
    resource_type = "AWS::AppSync::FunctionConfiguration"

    props = {
        "ApiId": (str, True),
        "DataSourceName": (str, True),
        "Description": (str, False),
        "FunctionVersion": (str, True),
        "Name": (str, True),
        "RequestMappingTemplate": (str, False),
        "RequestMappingTemplateS3Location": (str, False),
        "ResponseMappingTemplate": (str, False),
        "ResponseMappingTemplateS3Location": (str, False),
    }


class CognitoUserPoolConfig(AWSProperty):
    props = {
        "AppIdClientRegex": (str, False),
        "AwsRegion": (str, False),
        "UserPoolId": (str, False),
    }


class OpenIDConnectConfig(AWSProperty):
    props = {
        "AuthTTL": (double, False),
        "ClientId": (str, False),
        "IatTTL": (double, False),
        "Issuer": (str, False),
    }


class AdditionalAuthenticationProvider(AWSProperty):
    props = {
        "AuthenticationType": (str, True),
        "OpenIDConnectConfig": (OpenIDConnectConfig, False),
        "UserPoolConfig": (CognitoUserPoolConfig, False),
    }


class LambdaAuthorizerConfig(AWSProperty):
    props = {
        "AuthorizerResultTtlInSeconds": (double, False),
        "AuthorizerUri": (str, False),
        "IdentityValidationExpression": (str, False),
    }


class LogConfig(AWSProperty):
    props = {
        "CloudWatchLogsRoleArn": (str, False),
        "ExcludeVerboseContent": (boolean, False),
        "FieldLogLevel": (str, False),
    }


class UserPoolConfig(AWSProperty):
    props = {
        "AppIdClientRegex": (str, False),
        "AwsRegion": (str, False),
        "DefaultAction": (str, False),
        "UserPoolId": (str, False),
    }


class GraphQLApi(AWSObject):
    resource_type = "AWS::AppSync::GraphQLApi"

    props = {
        "AdditionalAuthenticationProviders": (
            [AdditionalAuthenticationProvider],
            False,
        ),
        "AuthenticationType": (str, True),
        "LambdaAuthorizerConfig": (LambdaAuthorizerConfig, False),
        "LogConfig": (LogConfig, False),
        "Name": (str, True),
        "OpenIDConnectConfig": (OpenIDConnectConfig, False),
        "Tags": (Tags, False),
        "UserPoolConfig": (UserPoolConfig, False),
        "XrayEnabled": (boolean, False),
    }


class GraphQLSchema(AWSObject):
    resource_type = "AWS::AppSync::GraphQLSchema"

    props = {
        "ApiId": (str, True),
        "Definition": (str, False),
        "DefinitionS3Location": (str, False),
    }


class CachingConfig(AWSProperty):
    props = {
        "CachingKeys": ([str], False),
        "Ttl": (double, False),
    }


class PipelineConfig(AWSProperty):
    props = {
        "Functions": ([str], False),
    }


class LambdaConflictHandlerConfig(AWSProperty):
    props = {
        "LambdaConflictHandlerArn": (str, False),
    }


class SyncConfig(AWSProperty):
    props = {
        "ConflictDetection": (str, True),
        "ConflictHandler": (str, False),
        "LambdaConflictHandlerConfig": (LambdaConflictHandlerConfig, False),
    }


class Resolver(AWSObject):
    resource_type = "AWS::AppSync::Resolver"

    props = {
        "ApiId": (str, True),
        "CachingConfig": (CachingConfig, False),
        "DataSourceName": (str, False),
        "FieldName": (str, True),
        "Kind": (resolver_kind_validator, False),
        "PipelineConfig": (PipelineConfig, False),
        "RequestMappingTemplate": (str, False),
        "RequestMappingTemplateS3Location": (str, False),
        "ResponseMappingTemplate": (str, False),
        "ResponseMappingTemplateS3Location": (str, False),
        "SyncConfig": (SyncConfig, False),
        "TypeName": (str, True),
    }



# Converted from ElasticBeanstalk_Nodejs.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from awacs.aws import Action, Allow, PolicyDocument, Principal, Statement
from awacs.sts import AssumeRole

from troposphere import FindInMap, GetAtt, Join, Output, Parameter, Ref, Template
from troposphere.elasticbeanstalk import (
    Application,
    ApplicationVersion,
    ConfigurationTemplate,
    Environment,
    OptionSetting,
    SourceBundle,
)
from troposphere.iam import InstanceProfile
from troposphere.iam import PolicyType as IAMPolicy
from troposphere.iam import Role

t = Template()

t.set_version()

t.set_description(
    "AWS CloudFormation Sample Template ElasticBeanstalk_Nodejs_Sample: "
    "Configure and launch the AWS Elastic Beanstalk sample application. "
    "**WARNING** This template creates one or more Amazon EC2 instances. You "
    "will be billed for the AWS resources used if you create a stack from "
    "this template."
)

keyname = t.add_parameter(
    Parameter(
        "KeyName",
        Description="Name of an existing EC2 KeyPair to enable SSH access to "
        "the AWS Elastic Beanstalk instance",
        Type="AWS::EC2::KeyPair::KeyName",
        ConstraintDescription="must be the name of an existing EC2 KeyPair.",
    )
)

t.add_mapping(
    "Region2Principal",
    {
        "ap-northeast-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "ap-southeast-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "ap-southeast-2": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "cn-north-1": {
            "EC2Principal": "ec2.amazonaws.com.cn",
            "OpsWorksPrincipal": "opsworks.amazonaws.com.cn",
        },
        "eu-central-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "eu-west-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "sa-east-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "us-east-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "us-west-1": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
        "us-west-2": {
            "EC2Principal": "ec2.amazonaws.com",
            "OpsWorksPrincipal": "opsworks.amazonaws.com",
        },
    },
)

t.add_resource(
    Role(
        "WebServerRole",
        AssumeRolePolicyDocument=PolicyDocument(
            Statement=[
                Statement(
                    Effect=Allow,
                    Action=[AssumeRole],
                    Principal=Principal(
                        "Service",
                        [
                            FindInMap(
                                "Region2Principal", Ref("AWS::Region"), "EC2Principal"
                            )
                        ],
                    ),
                )
            ]
        ),
        Path="/",
    )
)

t.add_resource(
    IAMPolicy(
        "WebServerRolePolicy",
        PolicyName="WebServerRole",
        PolicyDocument=PolicyDocument(
            Statement=[
                Statement(Effect=Allow, NotAction=Action("iam", "*"), Resource=["*"])
            ]
        ),
        Roles=[Ref("WebServerRole")],
    )
)

t.add_resource(
    InstanceProfile("WebServerInstanceProfile", Path="/", Roles=[Ref("WebServerRole")])
)

t.add_resource(
    Application(
        "SampleApplication",
        Description="AWS Elastic Beanstalk Sample Node.js Application",
    )
)

t.add_resource(
    ApplicationVersion(
        "SampleApplicationVersion",
        Description="Version 1.0",
        ApplicationName=Ref("SampleApplication"),
        SourceBundle=SourceBundle(
            S3Bucket=Join("-", ["elasticbeanstalk-samples", Ref("AWS::Region")]),
            S3Key="nodejs-sample.zip",
        ),
    )
)

t.add_resource(
    ConfigurationTemplate(
        "SampleConfigurationTemplate",
        ApplicationName=Ref("SampleApplication"),
        Description="SSH access to Node.JS Application",
        SolutionStackName="64bit Amazon Linux 2014.03 v1.0.9 running Node.js",
        OptionSettings=[
            OptionSetting(
                Namespace="aws:autoscaling:launchconfiguration",
                OptionName="EC2KeyName",
                Value=Ref("KeyName"),
            ),
            OptionSetting(
                Namespace="aws:autoscaling:launchconfiguration",
                OptionName="IamInstanceProfile",
                Value=Ref("WebServerInstanceProfile"),
            ),
        ],
    )
)

t.add_resource(
    Environment(
        "SampleEnvironment",
        Description="AWS Elastic Beanstalk Environment running Sample Node.js "
        "Application",
        ApplicationName=Ref("SampleApplication"),
        TemplateName=Ref("SampleConfigurationTemplate"),
        VersionLabel=Ref("SampleApplicationVersion"),
    )
)

t.add_output(
    Output(
        "URL",
        Description="URL of the AWS Elastic Beanstalk Environment",
        Value=Join("", ["http://", GetAtt("SampleEnvironment", "EndpointURL")]),
    )
)

print(t.to_json())



# Copyright (c) 2014, Guillem Anguera <ganguera@gmail.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import integer


class StreamEncryption(AWSProperty):
    props = {
        "EncryptionType": (str, True),
        "KeyId": (str, True),
    }


class Stream(AWSObject):
    resource_type = "AWS::Kinesis::Stream"

    props = {
        "Name": (str, False),
        "RetentionPeriodHours": (integer, False),
        "ShardCount": (integer, True),
        "StreamEncryption": (StreamEncryption, False),
        "Tags": ((Tags, list), False),
    }


class StreamConsumer(AWSObject):
    resource_type = "AWS::Kinesis::StreamConsumer"

    props = {
        "ConsumerName": (str, True),
        "StreamARN": (str, True),
    }



import troposphere.msk as msk
from troposphere import Template

t = Template()

t.add_resource(
    msk.Cluster(
        "TestCluster",
        ClusterName="MyMskCluster",
        KafkaVersion="2.1.0",
        NumberOfBrokerNodes=3,
        EnhancedMonitoring="PER_BROKER",
        BrokerNodeGroupInfo=msk.BrokerNodeGroupInfo(
            BrokerAZDistribution="DEFAULT",
            InstanceType="kafka.m5.large",
            SecurityGroups=["sg-c73ebda3"],
            StorageInfo=msk.StorageInfo(
                EBSStorageInfo=msk.EBSStorageInfo(VolumeSize=100)
            ),
            ClientSubnets=[
                "subnet-ce49ff7bcd",
                "subnet-2541a68474",
                "subnet-1d6b6f39da",
            ],
        ),
        EncryptionInfo=msk.EncryptionInfo(
            EncryptionAtRest=msk.EncryptionAtRest(
                DataVolumeKMSKeyId="ReplaceWithKmsKeyArn"
            ),
            EncryptionInTransit=msk.EncryptionInTransit(
                ClientBroker="TLS",
                InCluster=True,
            ),
        ),
        ClientAuthentication=msk.ClientAuthentication(
            Tls=msk.Tls(CertificateAuthorityArnList=["ReplaceWithCAArn"])
        ),
        ConfigurationInfo=msk.ConfigurationInfo(
            Arn=(
                "arn:aws:kafka:us-east-1:123456789012:configuration/"
                "example-configuration-name/"
                "abcdabcd-1234-abcd-1234-abcd123e8e8e-1"
            ),
            Revision=1,
        ),
        Tags=dict(MyTagName="MyTagValue"),
    )
)

print(t.to_json())



# Converted from RDS_Snapshot_On_Delete.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import GetAtt, Join, Output, Template
from troposphere.rds import DBInstance

t = Template()

t.set_version("2010-09-09")

t.set_description(
    "AWS CloudFormation Sample Template RDS_Snapshot_On_Delete: Sample "
    "template showing how to create an RDS DBInstance that is snapshotted on "
    "stack deletion. **WARNING** This template creates an Amazon RDS database "
    "instance. When the stack is deleted a database snpshot will be left in "
    "your account. You will be billed for the AWS resources used if you "
    "create a stack from this template."
)
MyDB = t.add_resource(
    DBInstance(
        "MyDB",
        Engine="MySQL",
        MasterUsername="myName",
        MasterUserPassword="myPassword",
        AllocatedStorage="5",
        DBInstanceClass="db.m1.small",
        DBName="MyDatabase",
    )
)

JDBCConnectionString = t.add_output(
    Output(
        "JDBCConnectionString",
        Description="JDBC connection string for the database",
        Value=Join(
            "",
            [
                "jdbc:mysql://",
                GetAtt(MyDB, "Endpoint.Address"),
                ":",
                GetAtt(MyDB, "Endpoint.Port"),
                "/MyDatabase",
            ],
        ),
    )
)

print(t.to_json())



from troposphere import Template
from troposphere.certificatemanager import Certificate, DomainValidationOption

t = Template()

t.add_resource(
    Certificate(
        "mycert",
        DomainName="example.com",
        DomainValidationOptions=[
            DomainValidationOption(
                DomainName="example.com",
                ValidationDomain="example.com",
            ),
        ],
        Tags=[
            {"Key": "tag-key", "Value": "tag-value"},
        ],
    )
)

print(t.to_json())



from awacs.aws import Allow, PolicyDocument, Principal, Statement
from awacs.sts import AssumeRole

from troposphere import GetAtt, Tags, Template
from troposphere.dlm import (
    CreateRule,
    LifecyclePolicy,
    PolicyDetails,
    RetainRule,
    Schedule,
)
from troposphere.iam import Role

t = Template()
t.set_version("2010-09-09")

dlm_role = t.add_resource(
    Role(
        "DlmRole",
        AssumeRolePolicyDocument=PolicyDocument(
            Statement=[
                Statement(
                    Effect=Allow,
                    Action=[AssumeRole],
                    Principal=Principal("Service", ["ec2.amazonaws.com"]),
                )
            ]
        ),
    )
)

lifecycle_policy = t.add_resource(
    LifecyclePolicy(
        "LifecyclePolicy",
        Description="Daily backup",
        State="ENABLED",
        ExecutionRoleArn=GetAtt(dlm_role, "Arn"),
        PolicyDetails=PolicyDetails(
            ResourceTypes=["VOLUME"],
            TargetTags=Tags(
                Backup="True",
            ),
            Schedules=[
                Schedule(
                    Name="Daily Snapshots",
                    TagsToAdd=Tags(
                        type="DailySnapshot",
                    ),
                    CreateRule=CreateRule(
                        Interval=12, IntervalUnit="HOURS", Times=["13:00"]
                    ),
                    RetainRule=RetainRule(Count=1),
                    CopyTags=True,
                )
            ],
        ),
    )
)

print(t.to_json())



# Converted from S3_Bucket.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import Output, Ref, Template
from troposphere.s3 import Bucket, PublicRead

t = Template()

t.set_description(
    "AWS CloudFormation Sample Template S3_Bucket: Sample template showing "
    "how to create a publicly accessible S3 bucket. "
    "**WARNING** This template creates an Amazon S3 Bucket. "
    "You will be billed for the AWS resources used if you create "
    "a stack from this template."
)

s3bucket = t.add_resource(
    Bucket(
        "S3Bucket",
        AccessControl=PublicRead,
    )
)

t.add_output(
    Output(
        "BucketName",
        Value=Ref(s3bucket),
        Description="Name of S3 bucket to hold website content",
    )
)

print(t.to_json())



# Copyright (c) 2012-2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 18.6.0


from . import AWSObject


class RepositoryAssociation(AWSObject):
    resource_type = "AWS::CodeGuruReviewer::RepositoryAssociation"

    props = {
        "ConnectionArn": (str, False),
        "Name": (str, True),
        "Owner": (str, False),
        "Type": (str, True),
    }



import troposphere.emr as emr
import troposphere.iam as iam
from troposphere import Equals, If, Join, Not, Parameter, Ref, Tags, Template
from troposphere.constants import KEY_PAIR_NAME, M4_LARGE, NUMBER, SUBNET_ID

scaling_policy = emr.SimpleScalingPolicyConfiguration(
    AdjustmentType="EXACT_CAPACITY", ScalingAdjustment="1", CoolDown="300"
)


kms_key = "arn:aws:kms:us-east-1:123456789012:key/1234-1234-1234-1234-1234"

security_configuration = {
    "EncryptionConfiguration": {
        "EnableInTransitEncryption": True,
        "InTransitEncryptionConfiguration": {
            "TLSCertificateConfiguration": {
                "CertificateProviderType": "PEM",
                "S3Object": "s3://MyConfigStore/artifacts/MyCerts.zip",
            }
        },
        "EnableAtRestEncryption": True,
        "AtRestEncryptionConfiguration": {
            "S3EncryptionConfiguration": {
                "EncryptionMode": "SSE-KMS",
                "AwsKmsKey": kms_key,
            },
            "LocalDiskEncryptionConfiguration": {
                "EncryptionKeyProviderType": "AwsKms",
                "AwsKmsKey": kms_key,
            },
        },
    }
}


def generate_rules(rules_name):
    global emr, scaling_policy

    rules = [
        emr.ScalingRule(
            Name=rules_name,
            Description="%s rules" % rules_name,
            Action=emr.ScalingAction(
                Market="ON_DEMAND", SimpleScalingPolicyConfiguration=scaling_policy
            ),
            Trigger=emr.ScalingTrigger(
                CloudWatchAlarmDefinition=emr.CloudWatchAlarmDefinition(
                    ComparisonOperator="GREATER_THAN",
                    EvaluationPeriods="120",
                    MetricName="TestMetric",
                    Namespace="AWS/ElasticMapReduce",
                    Period="300",
                    Statistic="AVERAGE",
                    Threshold="50",
                    Unit="PERCENT",
                    Dimensions=[
                        emr.MetricDimension(
                            "my.custom.master.property", "my.custom.master.value"
                        )
                    ],
                )
            ),
        )
    ]
    return rules


template = Template()
template.set_description("Sample CloudFormation template for creating an EMR cluster")

keyname = template.add_parameter(
    Parameter(
        "KeyName",
        Description="Name of an existing EC2 KeyPair to enable SSH " "to the instances",
        Type=KEY_PAIR_NAME,
    )
)

subnet = template.add_parameter(
    Parameter(
        "Subnet", Description="Subnet ID for creating the EMR cluster", Type=SUBNET_ID
    )
)

spot = template.add_parameter(
    Parameter(
        "SpotPrice",
        Description="Spot price (or use 0 for 'on demand' instance)",
        Type=NUMBER,
        Default="0.1",
    )
)

withSpotPrice = "WithSpotPrice"
template.add_condition(withSpotPrice, Not(Equals(Ref(spot), "0")))

gcTimeRatio = template.add_parameter(
    Parameter(
        "GcTimeRatioValue",
        Description="Hadoop name node garbage collector time ratio",
        Type=NUMBER,
        Default="19",
    )
)

# IAM roles required by EMR

emr_service_role = template.add_resource(
    iam.Role(
        "EMRServiceRole",
        AssumeRolePolicyDocument={
            "Statement": [
                {
                    "Effect": "Allow",
                    "Principal": {"Service": ["elasticmapreduce.amazonaws.com"]},
                    "Action": ["sts:AssumeRole"],
                }
            ]
        },
        ManagedPolicyArns=[
            "arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceRole"
        ],
    )
)

emr_autoscaling_role = "EMR_AutoScaling_DefaultRole"

emr_job_flow_role = template.add_resource(
    iam.Role(
        "EMRJobFlowRole",
        AssumeRolePolicyDocument={
            "Statement": [
                {
                    "Effect": "Allow",
                    "Principal": {"Service": ["ec2.amazonaws.com"]},
                    "Action": ["sts:AssumeRole"],
                }
            ]
        },
        ManagedPolicyArns=[
            "arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceforEC2Role"
        ],
    )
)

emr_instance_profile = template.add_resource(
    iam.InstanceProfile("EMRInstanceProfile", Roles=[Ref(emr_job_flow_role)])
)

# EMR Cluster Resource

security_config = template.add_resource(
    emr.SecurityConfiguration(
        "EMRSecurityConfiguration",
        Name="EMRSampleClusterSecurityConfiguration",
        SecurityConfiguration=security_configuration,
    )
)

cluster = template.add_resource(
    emr.Cluster(
        "EMRSampleCluster",
        Name="EMR Sample Cluster",
        ReleaseLabel="emr-4.4.0",
        SecurityConfiguration=Ref(security_config),
        BootstrapActions=[
            emr.BootstrapActionConfig(
                Name="Dummy bootstrap action",
                ScriptBootstrapAction=emr.ScriptBootstrapActionConfig(
                    Path="file:/usr/share/aws/emr/scripts/install-hue",
                    Args=["dummy", "parameter"],
                ),
            )
        ],
        Configurations=[
            emr.Configuration(
                Classification="core-site",
                ConfigurationProperties={"hadoop.security.groups.cache.secs": "250"},
            ),
            emr.Configuration(
                Classification="mapred-site",
                ConfigurationProperties={
                    "mapred.tasktracker.map.tasks.maximum": "2",
                    "mapreduce.map.sort.spill.percent": "90",
                    "mapreduce.tasktracker.reduce.tasks.maximum": "5",
                },
            ),
            emr.Configuration(
                Classification="hadoop-env",
                Configurations=[
                    emr.Configuration(
                        Classification="export",
                        ConfigurationProperties={
                            "HADOOP_DATANODE_HEAPSIZE": "2048",
                            "HADOOP_NAMENODE_OPTS": Join(
                                "", ["-XX:GCTimeRatio=", Ref(gcTimeRatio)]
                            ),
                        },
                    )
                ],
            ),
        ],
        JobFlowRole=Ref(emr_instance_profile),
        ServiceRole=Ref(emr_service_role),
        AutoScalingRole=emr_autoscaling_role,
        Instances=emr.JobFlowInstancesConfig(
            Ec2KeyName=Ref(keyname),
            Ec2SubnetId=Ref(subnet),
            MasterInstanceGroup=emr.InstanceGroupConfigProperty(
                Name="Master Instance",
                InstanceCount="1",
                InstanceType=M4_LARGE,
                Market="ON_DEMAND",
                AutoScalingPolicy=emr.AutoScalingPolicy(
                    Constraints=emr.ScalingConstraints(
                        MinCapacity="1", MaxCapacity="3"
                    ),
                    Rules=generate_rules("MasterAutoScalingPolicy"),
                ),
            ),
            CoreInstanceGroup=emr.InstanceGroupConfigProperty(
                Name="Core Instance",
                BidPrice=If(withSpotPrice, Ref(spot), Ref("AWS::NoValue")),
                Market=If(withSpotPrice, "SPOT", "ON_DEMAND"),
                AutoScalingPolicy=emr.AutoScalingPolicy(
                    Constraints=emr.ScalingConstraints(
                        MinCapacity="1", MaxCapacity="3"
                    ),
                    Rules=generate_rules("CoreAutoScalingPolicy"),
                ),
                EbsConfiguration=emr.EbsConfiguration(
                    EbsBlockDeviceConfigs=[
                        emr.EbsBlockDeviceConfigs(
                            VolumeSpecification=emr.VolumeSpecification(
                                SizeInGB="10", VolumeType="gp2"
                            ),
                            VolumesPerInstance="1",
                        )
                    ],
                    EbsOptimized="true",
                ),
                InstanceCount="1",
                InstanceType=M4_LARGE,
            ),
        ),
        Applications=[
            emr.Application(Name="Hadoop"),
            emr.Application(Name="Hive"),
            emr.Application(Name="Mahout"),
            emr.Application(Name="Pig"),
            emr.Application(Name="Spark"),
        ],
        VisibleToAllUsers="true",
        Tags=Tags(Name="EMR Sample Cluster"),
    )
)

step = template.add_resource(
    emr.Step(
        "TestStep",
        Name="TestStep",
        ActionOnFailure="CONTINUE",
        HadoopJarStep=emr.HadoopJarStepConfig(
            Args=["5", "10"],
            Jar="s3://emr-cfn-test/hadoop-mapreduce-examples-2.6.0.jar",
            MainClass="pi",
            StepProperties=[emr.KeyValue("my.custom.property", "my.custom.value")],
        ),
        JobFlowId=Ref(cluster),
    )
)

print(template.to_json())



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.3.0


from . import AWSObject, AWSProperty


class GeofenceCollection(AWSObject):
    resource_type = "AWS::Location::GeofenceCollection"

    props = {
        "CollectionName": (str, True),
        "Description": (str, False),
        "KmsKeyId": (str, False),
        "PricingPlan": (str, True),
        "PricingPlanDataSource": (str, False),
    }


class MapConfiguration(AWSProperty):
    props = {
        "Style": (str, True),
    }


class Map(AWSObject):
    resource_type = "AWS::Location::Map"

    props = {
        "Configuration": (MapConfiguration, True),
        "Description": (str, False),
        "MapName": (str, True),
        "PricingPlan": (str, True),
    }


class DataSourceConfiguration(AWSProperty):
    props = {
        "IntendedUse": (str, False),
    }


class PlaceIndex(AWSObject):
    resource_type = "AWS::Location::PlaceIndex"

    props = {
        "DataSource": (str, True),
        "DataSourceConfiguration": (DataSourceConfiguration, False),
        "Description": (str, False),
        "IndexName": (str, True),
        "PricingPlan": (str, True),
    }


class RouteCalculator(AWSObject):
    resource_type = "AWS::Location::RouteCalculator"

    props = {
        "CalculatorName": (str, True),
        "DataSource": (str, True),
        "Description": (str, False),
        "PricingPlan": (str, True),
    }


class Tracker(AWSObject):
    resource_type = "AWS::Location::Tracker"

    props = {
        "Description": (str, False),
        "KmsKeyId": (str, False),
        "PricingPlan": (str, True),
        "PricingPlanDataSource": (str, False),
        "TrackerName": (str, True),
    }


class TrackerConsumer(AWSObject):
    resource_type = "AWS::Location::TrackerConsumer"

    props = {
        "ConsumerArn": (str, True),
        "TrackerName": (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class Connector(AWSProperty):
    props = {
        "ConnectorArn": (str, True),
        "Id": (str, True),
        "Parameters": (dict, False),
    }


class ConnectorDefinitionVersion(AWSProperty):
    props = {
        "Connectors": ([Connector], True),
    }


class ConnectorDefinition(AWSObject):
    resource_type = "AWS::Greengrass::ConnectorDefinition"

    props = {
        "InitialVersion": (ConnectorDefinitionVersion, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class ConnectorDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::ConnectorDefinitionVersion"

    props = {
        "ConnectorDefinitionId": (str, True),
        "Connectors": ([Connector], True),
    }


class Core(AWSProperty):
    props = {
        "CertificateArn": (str, True),
        "Id": (str, True),
        "SyncShadow": (boolean, False),
        "ThingArn": (str, True),
    }


class CoreDefinitionVersion(AWSProperty):
    props = {
        "Cores": ([Core], True),
    }


class CoreDefinition(AWSObject):
    resource_type = "AWS::Greengrass::CoreDefinition"

    props = {
        "InitialVersion": (CoreDefinitionVersion, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class CoreDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::CoreDefinitionVersion"

    props = {
        "CoreDefinitionId": (str, True),
        "Cores": ([Core], True),
    }


class Device(AWSProperty):
    props = {
        "CertificateArn": (str, True),
        "Id": (str, True),
        "SyncShadow": (boolean, False),
        "ThingArn": (str, True),
    }


class DeviceDefinitionVersion(AWSProperty):
    props = {
        "Devices": ([Device], True),
    }


class DeviceDefinition(AWSObject):
    resource_type = "AWS::Greengrass::DeviceDefinition"

    props = {
        "InitialVersion": (DeviceDefinitionVersion, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class DeviceDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::DeviceDefinitionVersion"

    props = {
        "DeviceDefinitionId": (str, True),
        "Devices": ([Device], True),
    }


class RunAs(AWSProperty):
    props = {
        "Gid": (integer, False),
        "Uid": (integer, False),
    }


class Execution(AWSProperty):
    props = {
        "IsolationMode": (str, False),
        "RunAs": (RunAs, False),
    }


class DefaultConfig(AWSProperty):
    props = {
        "Execution": (Execution, True),
    }


class ResourceAccessPolicy(AWSProperty):
    props = {
        "Permission": (str, False),
        "ResourceId": (str, True),
    }


class Environment(AWSProperty):
    props = {
        "AccessSysfs": (boolean, False),
        "Execution": (Execution, False),
        "ResourceAccessPolicies": ([ResourceAccessPolicy], False),
        "Variables": (dict, False),
    }


class FunctionConfiguration(AWSProperty):
    props = {
        "EncodingType": (str, False),
        "Environment": (Environment, False),
        "ExecArgs": (str, False),
        "Executable": (str, False),
        "MemorySize": (integer, False),
        "Pinned": (boolean, False),
        "Timeout": (integer, False),
    }


class Function(AWSProperty):
    props = {
        "FunctionArn": (str, True),
        "FunctionConfiguration": (FunctionConfiguration, True),
        "Id": (str, True),
    }


class FunctionDefinitionVersion(AWSProperty):
    props = {
        "DefaultConfig": (DefaultConfig, False),
        "Functions": ([Function], True),
    }


class FunctionDefinition(AWSObject):
    resource_type = "AWS::Greengrass::FunctionDefinition"

    props = {
        "InitialVersion": (FunctionDefinitionVersion, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class FunctionDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::FunctionDefinitionVersion"

    props = {
        "DefaultConfig": (DefaultConfig, False),
        "FunctionDefinitionId": (str, True),
        "Functions": ([Function], True),
    }


class GroupVersion(AWSProperty):
    props = {
        "ConnectorDefinitionVersionArn": (str, False),
        "CoreDefinitionVersionArn": (str, False),
        "DeviceDefinitionVersionArn": (str, False),
        "FunctionDefinitionVersionArn": (str, False),
        "LoggerDefinitionVersionArn": (str, False),
        "ResourceDefinitionVersionArn": (str, False),
        "SubscriptionDefinitionVersionArn": (str, False),
    }


class Group(AWSObject):
    resource_type = "AWS::Greengrass::Group"

    props = {
        "InitialVersion": (GroupVersion, False),
        "Name": (str, True),
        "RoleArn": (str, False),
        "Tags": (dict, False),
    }


class GroupVersion(AWSObject):
    resource_type = "AWS::Greengrass::GroupVersion"

    props = {
        "ConnectorDefinitionVersionArn": (str, False),
        "CoreDefinitionVersionArn": (str, False),
        "DeviceDefinitionVersionArn": (str, False),
        "FunctionDefinitionVersionArn": (str, False),
        "GroupId": (str, True),
        "LoggerDefinitionVersionArn": (str, False),
        "ResourceDefinitionVersionArn": (str, False),
        "SubscriptionDefinitionVersionArn": (str, False),
    }


class Logger(AWSProperty):
    props = {
        "Component": (str, True),
        "Id": (str, True),
        "Level": (str, True),
        "Space": (integer, False),
        "Type": (str, True),
    }


class LoggerDefinitionVersion(AWSProperty):
    props = {
        "Loggers": ([Logger], True),
    }


class LoggerDefinition(AWSObject):
    resource_type = "AWS::Greengrass::LoggerDefinition"

    props = {
        "InitialVersion": (LoggerDefinitionVersion, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class LoggerDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::LoggerDefinitionVersion"

    props = {
        "LoggerDefinitionId": (str, True),
        "Loggers": ([Logger], True),
    }


class GroupOwnerSetting(AWSProperty):
    props = {
        "AutoAddGroupOwner": (boolean, True),
        "GroupOwner": (str, False),
    }


class LocalDeviceResourceData(AWSProperty):
    props = {
        "GroupOwnerSetting": (GroupOwnerSetting, False),
        "SourcePath": (str, True),
    }


class LocalVolumeResourceData(AWSProperty):
    props = {
        "DestinationPath": (str, True),
        "GroupOwnerSetting": (GroupOwnerSetting, False),
        "SourcePath": (str, True),
    }


class ResourceDownloadOwnerSetting(AWSProperty):
    props = {
        "GroupOwner": (str, True),
        "GroupPermission": (str, True),
    }


class S3MachineLearningModelResourceData(AWSProperty):
    props = {
        "DestinationPath": (str, True),
        "OwnerSetting": (ResourceDownloadOwnerSetting, False),
        "S3Uri": (str, True),
    }


class SageMakerMachineLearningModelResourceData(AWSProperty):
    props = {
        "DestinationPath": (str, True),
        "OwnerSetting": (ResourceDownloadOwnerSetting, False),
        "SageMakerJobArn": (str, True),
    }


class SecretsManagerSecretResourceData(AWSProperty):
    props = {
        "ARN": (str, True),
        "AdditionalStagingLabelsToDownload": ([str], False),
    }


class ResourceDataContainer(AWSProperty):
    props = {
        "LocalDeviceResourceData": (LocalDeviceResourceData, False),
        "LocalVolumeResourceData": (LocalVolumeResourceData, False),
        "S3MachineLearningModelResourceData": (
            S3MachineLearningModelResourceData,
            False,
        ),
        "SageMakerMachineLearningModelResourceData": (
            SageMakerMachineLearningModelResourceData,
            False,
        ),
        "SecretsManagerSecretResourceData": (SecretsManagerSecretResourceData, False),
    }


class ResourceInstance(AWSProperty):
    props = {
        "Id": (str, True),
        "Name": (str, True),
        "ResourceDataContainer": (ResourceDataContainer, True),
    }


class ResourceDefinitionVersion(AWSProperty):
    props = {
        "Resources": ([ResourceInstance], True),
    }


class ResourceDefinition(AWSObject):
    resource_type = "AWS::Greengrass::ResourceDefinition"

    props = {
        "InitialVersion": (ResourceDefinitionVersion, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class ResourceDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::ResourceDefinitionVersion"

    props = {
        "ResourceDefinitionId": (str, True),
        "Resources": ([ResourceInstance], True),
    }


class Subscription(AWSProperty):
    props = {
        "Id": (str, True),
        "Source": (str, True),
        "Subject": (str, True),
        "Target": (str, True),
    }


class SubscriptionDefinitionVersionProperty(AWSProperty):
    props = {
        "Subscriptions": ([Subscription], True),
    }


class SubscriptionDefinition(AWSObject):
    resource_type = "AWS::Greengrass::SubscriptionDefinition"

    props = {
        "InitialVersion": (SubscriptionDefinitionVersionProperty, False),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class SubscriptionDefinitionVersion(AWSObject):
    resource_type = "AWS::Greengrass::SubscriptionDefinitionVersion"

    props = {
        "SubscriptionDefinitionId": (str, True),
        "Subscriptions": ([Subscription], True),
    }



# Copyright (c) 2012-2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 18.6.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class Alarm(AWSProperty):
    props = {
        "AlarmName": (str, True),
        "Severity": (str, False),
    }


class AlarmMetric(AWSProperty):
    props = {
        "AlarmMetricName": (str, True),
    }


class Log(AWSProperty):
    props = {
        "Encoding": (str, False),
        "LogGroupName": (str, False),
        "LogPath": (str, False),
        "LogType": (str, True),
        "PatternSet": (str, False),
    }


class WindowsEvent(AWSProperty):
    props = {
        "EventLevels": ([str], True),
        "EventName": (str, True),
        "LogGroupName": (str, True),
        "PatternSet": (str, False),
    }


class ConfigurationDetails(AWSProperty):
    props = {
        "AlarmMetrics": ([AlarmMetric], False),
        "Alarms": ([Alarm], False),
        "Logs": ([Log], False),
        "WindowsEvents": ([WindowsEvent], False),
    }


class SubComponentConfigurationDetails(AWSProperty):
    props = {
        "AlarmMetrics": ([AlarmMetric], False),
        "Logs": ([Log], False),
        "WindowsEvents": ([WindowsEvent], False),
    }


class SubComponentTypeConfiguration(AWSProperty):
    props = {
        "SubComponentConfigurationDetails": (SubComponentConfigurationDetails, True),
        "SubComponentType": (str, True),
    }


class ComponentConfiguration(AWSProperty):
    props = {
        "ConfigurationDetails": (ConfigurationDetails, False),
        "SubComponentTypeConfigurations": ([SubComponentTypeConfiguration], False),
    }


class ComponentMonitoringSetting(AWSProperty):
    props = {
        "ComponentARN": (str, False),
        "ComponentConfigurationMode": (str, False),
        "ComponentName": (str, False),
        "CustomComponentConfiguration": (ComponentConfiguration, False),
        "DefaultOverwriteComponentConfiguration": (ComponentConfiguration, False),
        "Tier": (str, False),
    }


class CustomComponent(AWSProperty):
    props = {
        "ComponentName": (str, True),
        "ResourceList": ([str], True),
    }


class LogPattern(AWSProperty):
    props = {
        "Pattern": (str, True),
        "PatternName": (str, True),
        "Rank": (integer, True),
    }


class LogPatternSet(AWSProperty):
    props = {
        "LogPatterns": ([LogPattern], True),
        "PatternSetName": (str, True),
    }


class Application(AWSObject):
    resource_type = "AWS::ApplicationInsights::Application"

    props = {
        "AutoConfigurationEnabled": (boolean, False),
        "CWEMonitorEnabled": (boolean, False),
        "ComponentMonitoringSettings": ([ComponentMonitoringSetting], False),
        "CustomComponents": ([CustomComponent], False),
        "LogPatternSets": ([LogPatternSet], False),
        "OpsCenterEnabled": (boolean, False),
        "OpsItemSNSTopicArn": (str, False),
        "ResourceGroupName": (str, True),
        "Tags": (Tags, False),
    }



# Converted from s3_processor located at:
# https://github.com/awslabs/serverless-application-model/blob/dbc54b5d0cd31bf5cebd16d765b74aee9eb34641/examples/2016-10-31/s3_processor/template.yaml

from troposphere import Ref, Template
from troposphere.s3 import Bucket
from troposphere.serverless import Function, S3Event

t = Template()

t.set_description(
    "A function is triggered off an upload to a bucket. It logs the content "
    "type of the uploaded object."
)

t.set_transform("AWS::Serverless-2016-10-31")


s3_bucket = t.add_resource(Bucket("Bucket"))

t.add_resource(
    Function(
        "ProcessorFunction",
        Handler="index.handler",
        Runtime="nodejs4.3",
        CodeUri="s3://<bucket>/s3_processor.zip",
        Policies="AmazonS3ReadOnlyAccess",
        Events={
            "PhotoUpload": S3Event(
                "PhotoUpload", Bucket=Ref(s3_bucket), Events=["s3:ObjectCreated:*"]
            )
        },
    )
)

print(t.to_json())



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 35.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import integer


class Repository(AWSProperty):
    props = {
        "PathComponent": (str, True),
        "RepositoryUrl": (str, True),
    }


class EnvironmentEC2(AWSObject):
    resource_type = "AWS::Cloud9::EnvironmentEC2"

    props = {
        "AutomaticStopTimeMinutes": (integer, False),
        "ConnectionType": (str, False),
        "Description": (str, False),
        "ImageId": (str, False),
        "InstanceType": (str, True),
        "Name": (str, False),
        "OwnerArn": (str, False),
        "Repositories": ([Repository], False),
        "SubnetId": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.1.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, double, integer


class InsightsConfiguration(AWSProperty):
    props = {
        "InsightsEnabled": (boolean, False),
        "NotificationsEnabled": (boolean, False),
    }


class Group(AWSObject):
    resource_type = "AWS::XRay::Group"

    props = {
        "FilterExpression": (str, False),
        "GroupName": (str, False),
        "InsightsConfiguration": (InsightsConfiguration, False),
        "Tags": (Tags, False),
    }


class SamplingRule(AWSProperty):
    props = {
        "Attributes": (dict, False),
        "FixedRate": (double, False),
        "HTTPMethod": (str, False),
        "Host": (str, False),
        "Priority": (integer, False),
        "ReservoirSize": (integer, False),
        "ResourceARN": (str, False),
        "RuleARN": (str, False),
        "RuleName": (str, False),
        "ServiceName": (str, False),
        "ServiceType": (str, False),
        "URLPath": (str, False),
        "Version": (integer, False),
    }


class SamplingRuleRecord(AWSProperty):
    props = {
        "CreatedAt": (str, False),
        "ModifiedAt": (str, False),
        "SamplingRule": (SamplingRule, False),
    }


class SamplingRuleUpdate(AWSProperty):
    props = {
        "Attributes": (dict, False),
        "FixedRate": (double, False),
        "HTTPMethod": (str, False),
        "Host": (str, False),
        "Priority": (integer, False),
        "ReservoirSize": (integer, False),
        "ResourceARN": (str, False),
        "RuleARN": (str, False),
        "RuleName": (str, False),
        "ServiceName": (str, False),
        "ServiceType": (str, False),
        "URLPath": (str, False),
    }



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import resourcequery_type


class TagFilter(AWSProperty):
    props = {
        "Key": (str, False),
        "Values": ([str], False),
    }


class Query(AWSProperty):
    props = {
        "ResourceTypeFilters": ([str], False),
        "StackIdentifier": (str, False),
        "TagFilters": ([TagFilter], False),
    }


class ResourceQuery(AWSProperty):
    props = {
        "Query": (Query, False),
        "Type": (resourcequery_type, False),
    }


class Group(AWSObject):
    resource_type = "AWS::ResourceGroups::Group"

    props = {
        "Description": (str, False),
        "Name": (str, True),
        "ResourceQuery": (ResourceQuery, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class UpdateError(AWSProperty):
    props = {
        "ErrorCode": (str, False),
        "ErrorMessage": (str, False),
    }


class LastUpdate(AWSProperty):
    props = {
        "CreatedAt": (str, False),
        "Error": (UpdateError, False),
        "Status": (str, False),
    }


class ModuleLoggingConfiguration(AWSProperty):
    props = {
        "CloudWatchLogGroupArn": (str, False),
        "Enabled": (boolean, False),
        "LogLevel": (str, False),
    }


class LoggingConfiguration(AWSProperty):
    props = {
        "DagProcessingLogs": (ModuleLoggingConfiguration, False),
        "SchedulerLogs": (ModuleLoggingConfiguration, False),
        "TaskLogs": (ModuleLoggingConfiguration, False),
        "WebserverLogs": (ModuleLoggingConfiguration, False),
        "WorkerLogs": (ModuleLoggingConfiguration, False),
    }


class NetworkConfiguration(AWSProperty):
    props = {
        "SecurityGroupIds": (list, True),
        "SubnetIds": (list, True),
    }


class Environment(AWSObject):
    resource_type = "AWS::MWAA::Environment"

    props = {
        "AirflowConfigurationOptions": (dict, False),
        "AirflowVersion": (str, False),
        "DagS3Path": (str, False),
        "EnvironmentClass": (str, False),
        "ExecutionRoleArn": (str, False),
        "KmsKey": (str, False),
        "LoggingConfiguration": (LoggingConfiguration, False),
        "MaxWorkers": (integer, False),
        "MinWorkers": (integer, False),
        "Name": (str, True),
        "NetworkConfiguration": (NetworkConfiguration, False),
        "PluginsS3ObjectVersion": (str, False),
        "PluginsS3Path": (str, False),
        "RequirementsS3ObjectVersion": (str, False),
        "RequirementsS3Path": (str, False),
        "Schedulers": (integer, False),
        "SourceBucketArn": (str, False),
        "Tags": (Tags, False),
        "WebserverAccessMode": (str, False),
        "WeeklyMaintenanceWindowStart": (str, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 10.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean


class Filter(AWSProperty):
    props = {
        "Contains": ([str], False),
        "Eq": ([str], False),
        "Exists": (boolean, False),
        "Neq": ([str], False),
        "Property": (str, True),
    }


class ArchiveRule(AWSProperty):
    props = {
        "Filter": ([Filter], True),
        "RuleName": (str, True),
    }


class Analyzer(AWSObject):
    resource_type = "AWS::AccessAnalyzer::Analyzer"

    props = {
        "AnalyzerName": (str, False),
        "ArchiveRules": ([ArchiveRule], False),
        "Tags": (Tags, False),
        "Type": (str, True),
    }



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, If
from .validators import backup_vault_name, double, exactly_one, json_checker


class AdvancedBackupSettingResourceType(AWSProperty):
    props = {
        "BackupOptions": (dict, True),
        "ResourceType": (str, True),
    }


class LifecycleResourceType(AWSProperty):
    props = {
        "DeleteAfterDays": (double, False),
        "MoveToColdStorageAfterDays": (double, False),
    }


class CopyActionResourceType(AWSProperty):
    props = {
        "DestinationBackupVaultArn": (str, True),
        "Lifecycle": (LifecycleResourceType, False),
    }


class BackupRuleResourceType(AWSProperty):
    props = {
        "CompletionWindowMinutes": (double, False),
        "CopyActions": ([CopyActionResourceType], False),
        "Lifecycle": (LifecycleResourceType, False),
        "RecoveryPointTags": (dict, False),
        "RuleName": (str, True),
        "ScheduleExpression": (str, False),
        "StartWindowMinutes": (double, False),
        "TargetBackupVault": (str, True),
    }


class BackupPlanResourceType(AWSProperty):
    props = {
        "AdvancedBackupSettings": ([AdvancedBackupSettingResourceType], False),
        "BackupPlanName": (str, True),
        "BackupPlanRule": ([BackupRuleResourceType], True),
    }


class BackupPlan(AWSObject):
    resource_type = "AWS::Backup::BackupPlan"

    props = {
        "BackupPlan": (BackupPlanResourceType, True),
        "BackupPlanTags": (dict, False),
    }


class ConditionResourceType(AWSProperty):
    props = {
        "ConditionKey": (str, True),
        "ConditionType": (str, True),
        "ConditionValue": (str, True),
    }


class BackupSelectionResourceType(AWSProperty):
    props = {
        "IamRoleArn": (str, True),
        "ListOfTags": ([ConditionResourceType], False),
        "Resources": ([str], False),
        "SelectionName": (str, True),
    }

    def validate(self):
        conds = [
            "ListOfTags",
            "Resources",
        ]

        def check_if(names, props):
            validated = []
            for name in names:
                validated.append(name in props and isinstance(props[name], If))
            return all(validated)

        if check_if(conds, self.properties):
            return

        exactly_one(self.__class__.__name__, self.properties, conds)


class BackupSelection(AWSObject):
    resource_type = "AWS::Backup::BackupSelection"

    props = {
        "BackupPlanId": (str, True),
        "BackupSelection": (BackupSelectionResourceType, True),
    }


class NotificationObjectType(AWSProperty):
    props = {"BackupVaultEvents": ([str], True), "SNSTopicArn": (str, True)}


class BackupVault(AWSObject):
    resource_type = "AWS::Backup::BackupVault"

    props = {
        "AccessPolicy": (json_checker, False),
        "BackupVaultName": (backup_vault_name, True),
        "BackupVaultTags": (dict, False),
        "EncryptionKeyArn": (str, False),
        "Notifications": (NotificationObjectType, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import boolean


class Subscription(AWSProperty):
    props = {
        "Endpoint": (str, True),
        "Protocol": (str, True),
    }


class SubscriptionResource(AWSObject):
    resource_type = "AWS::SNS::Subscription"

    props = {
        "DeliveryPolicy": (dict, False),
        "Endpoint": (str, False),
        "FilterPolicy": (dict, False),
        "Protocol": (str, True),
        "RawMessageDelivery": (boolean, False),
        "RedrivePolicy": (dict, False),
        "Region": (str, False),
        "TopicArn": (str, True),
    }


class TopicPolicy(AWSObject):
    resource_type = "AWS::SNS::TopicPolicy"

    props = {
        "PolicyDocument": (policytypes, True),
        "Topics": (list, True),
    }


class Topic(AWSObject):
    resource_type = "AWS::SNS::Topic"

    props = {
        "ContentBasedDeduplication": (boolean, False),
        "DisplayName": (str, False),
        "FifoTopic": (boolean, False),
        "KmsMasterKeyId": (str, False),
        "Subscription": ([Subscription], False),
        "Tags": (Tags, False),
        "TopicName": (str, False),
    }



# Copyright (c) 2013, Bob Van Zant <bob@veznat.com>
# All rights reserved.
#
# See LICENSE file for full license.
import warnings

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import (
    boolean,
    double,
    integer,
    positive_integer,
    s3_bucket_name,
    s3_transfer_acceleration_status,
)

Private = "Private"
PublicRead = "PublicRead"
PublicReadWrite = "PublicReadWrite"
AuthenticatedRead = "AuthenticatedRead"
BucketOwnerRead = "BucketOwnerRead"
BucketOwnerFullControl = "BucketOwnerFullControl"
LogDeliveryWrite = "LogDeliveryWrite"


class PublicAccessBlockConfiguration(AWSProperty):
    props = {
        "BlockPublicAcls": (boolean, False),
        "BlockPublicPolicy": (boolean, False),
        "IgnorePublicAcls": (boolean, False),
        "RestrictPublicBuckets": (boolean, False),
    }


class VpcConfiguration(AWSProperty):
    props = {
        "VpcId": (str, False),
    }


class AccessPoint(AWSObject):
    resource_type = "AWS::S3::AccessPoint"

    props = {
        "Bucket": (str, True),
        "CreationDate": (str, False),
        "Name": (str, False),
        "NetworkOrigin": (str, False),
        "Policy": (policytypes, False),
        "PolicyStatus": (dict, False),
        "PublicAccessBlockConfiguration": (PublicAccessBlockConfiguration, False),
        "VpcConfiguration": (VpcConfiguration, False),
    }


class CorsRules(AWSProperty):
    props = {
        "AllowedHeaders": ([str], False),
        "AllowedMethods": ([str], True),
        "AllowedOrigins": ([str], True),
        "ExposedHeaders": ([str], False),
        "Id": (str, False),
        "MaxAge": (positive_integer, False),
    }


class CorsConfiguration(AWSProperty):
    props = {
        "CorsRules": ([CorsRules], True),
    }


class VersioningConfiguration(AWSProperty):
    props = {
        "Status": (str, False),
    }


class AccelerateConfiguration(AWSProperty):
    props = {
        "AccelerationStatus": (s3_transfer_acceleration_status, True),
    }


class RedirectAllRequestsTo(AWSProperty):
    props = {
        "HostName": (str, True),
        "Protocol": (str, False),
    }


class RedirectRule(AWSProperty):
    props = {
        "HostName": (str, False),
        "HttpRedirectCode": (str, False),
        "Protocol": (str, False),
        "ReplaceKeyPrefixWith": (str, False),
        "ReplaceKeyWith": (str, False),
    }


class RoutingRuleCondition(AWSProperty):
    props = {
        "HttpErrorCodeReturnedEquals": (str, False),
        "KeyPrefixEquals": (str, False),
    }


class RoutingRule(AWSProperty):
    props = {
        "RedirectRule": (RedirectRule, True),
        "RoutingRuleCondition": (RoutingRuleCondition, False),
    }


class WebsiteConfiguration(AWSProperty):
    props = {
        "IndexDocument": (str, False),
        "ErrorDocument": (str, False),
        "RedirectAllRequestsTo": (RedirectAllRequestsTo, False),
        "RoutingRules": ([RoutingRule], False),
    }


class LifecycleRuleTransition(AWSProperty):
    props = {
        "StorageClass": (str, True),
        "TransitionDate": (str, False),
        "TransitionInDays": (positive_integer, False),
    }


class AbortIncompleteMultipartUpload(AWSProperty):
    props = {
        "DaysAfterInitiation": (positive_integer, True),
    }


class NoncurrentVersionTransition(AWSProperty):
    props = {
        "StorageClass": (str, True),
        "TransitionInDays": (positive_integer, True),
    }


class TagFilter(AWSProperty):
    props = {
        "Key": (str, True),
        "Value": (str, True),
    }


class LifecycleRule(AWSProperty):
    props = {
        "AbortIncompleteMultipartUpload": (AbortIncompleteMultipartUpload, False),
        "ExpirationDate": (str, False),
        "ExpirationInDays": (positive_integer, False),
        "ExpiredObjectDeleteMarker": (boolean, False),
        "Id": (str, False),
        "NoncurrentVersionExpirationInDays": (positive_integer, False),
        "NoncurrentVersionTransition": (NoncurrentVersionTransition, False),
        "NoncurrentVersionTransitions": ([NoncurrentVersionTransition], False),
        "Prefix": (str, False),
        "Status": (str, True),
        "TagFilters": ([TagFilter], False),
        "Transition": (LifecycleRuleTransition, False),
        "Transitions": ([LifecycleRuleTransition], False),
    }

    def validate(self):
        if "Transition" in self.properties:
            if "Transitions" not in self.properties:
                # aws moved from a single transition to a list of them
                # and deprecated 'Transition', so let's just move it to
                # the new property and not annoy the user.
                self.properties["Transitions"] = [self.properties.pop("Transition")]
            else:
                raise ValueError(
                    'Cannot specify both "Transition" and "Transitions" '
                    "properties on S3 Bucket Lifecycle Rule. Please use "
                    '"Transitions" since the former has been deprecated.'
                )

        if "NoncurrentVersionTransition" in self.properties:
            if "NoncurrentVersionTransitions" not in self.properties:
                warnings.warn(
                    "NoncurrentVersionTransition has been deprecated in "
                    "favour of NoncurrentVersionTransitions."
                )
                # Translate the old transition format to the new format
                self.properties["NoncurrentVersionTransitions"] = [
                    self.properties.pop("NoncurrentVersionTransition")
                ]
            else:
                raise ValueError(
                    'Cannot specify both "NoncurrentVersionTransition" and '
                    '"NoncurrentVersionTransitions" properties on S3 Bucket '
                    "Lifecycle Rule. Please use "
                    '"NoncurrentVersionTransitions" since the former has been '
                    "deprecated."
                )

        if (
            "ExpirationInDays" in self.properties
            and "ExpirationDate" in self.properties
        ):
            raise ValueError(
                'Cannot specify both "ExpirationDate" and "ExpirationInDays"'
            )


class LifecycleConfiguration(AWSProperty):
    props = {
        "Rules": ([LifecycleRule], True),
    }


class LoggingConfiguration(AWSProperty):
    props = {
        "DestinationBucketName": (s3_bucket_name, False),
        "LogFilePrefix": (str, False),
    }


class Rules(AWSProperty):
    props = {"Name": (str, True), "Value": (str, True)}


class S3Key(AWSProperty):
    props = {"Rules": ([Rules], True)}


class Filter(AWSProperty):
    props = {"S3Key": (S3Key, True)}


class LambdaConfigurations(AWSProperty):
    props = {
        "Event": (str, True),
        "Filter": (Filter, False),
        "Function": (str, True),
    }


class QueueConfigurations(AWSProperty):
    props = {
        "Event": (str, True),
        "Filter": (Filter, False),
        "Queue": (str, True),
    }


class TopicConfigurations(AWSProperty):
    props = {
        "Event": (str, True),
        "Filter": (Filter, False),
        "Topic": (str, True),
    }


class MetricsConfiguration(AWSProperty):
    props = {
        "Id": (str, True),
        "Prefix": (str, False),
        "TagFilters": ([TagFilter], False),
    }


class NotificationConfiguration(AWSProperty):
    props = {
        "LambdaConfigurations": ([LambdaConfigurations], False),
        "QueueConfigurations": ([QueueConfigurations], False),
        "TopicConfigurations": ([TopicConfigurations], False),
    }


class OwnershipControlsRule(AWSProperty):
    props = {
        "ObjectOwnership": (str, False),
    }


class OwnershipControls(AWSProperty):
    props = {
        "Rules": ([OwnershipControlsRule], True),
    }


class DeleteMarkerReplication(AWSProperty):
    props = {
        "Status": (str, False),
    }


class AccessControlTranslation(AWSProperty):
    props = {
        "Owner": (str, True),
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        "ReplicaKmsKeyID": (str, True),
    }


class ReplicationTimeValue(AWSProperty):
    props = {
        "Minutes": (integer, True),
    }


class Metrics(AWSProperty):
    props = {
        "EventThreshold": (ReplicationTimeValue, False),
        "Status": (str, True),
    }


class ReplicationTime(AWSProperty):
    props = {
        "Status": (str, True),
        "Time": (ReplicationTimeValue, True),
    }


class ReplicationConfigurationRulesDestination(AWSProperty):
    props = {
        "AccessControlTranslation": (AccessControlTranslation, False),
        "Account": (str, False),
        "Bucket": (str, True),
        "EncryptionConfiguration": (EncryptionConfiguration, False),
        "Metrics": (Metrics, False),
        "ReplicationTime": (ReplicationTime, False),
        "StorageClass": (str, False),
    }


class ReplicationRuleAndOperator(AWSProperty):
    props = {
        "Prefix": (str, False),
        "TagFilters": ([TagFilter], False),
    }


class ReplicationRuleFilter(AWSProperty):
    props = {
        "And": (ReplicationRuleAndOperator, False),
        "Prefix": (str, False),
        "TagFilter": (TagFilter, False),
    }


class ReplicaModifications(AWSProperty):
    props = {
        "Status": (str, True),
    }


class SseKmsEncryptedObjects(AWSProperty):
    props = {
        "Status": (str, True),
    }


class SourceSelectionCriteria(AWSProperty):
    props = {
        "ReplicaModifications": (ReplicaModifications, False),
        "SseKmsEncryptedObjects": (SseKmsEncryptedObjects, False),
    }


class ReplicationConfigurationRules(AWSProperty):
    props = {
        "DeleteMarkerReplication": (DeleteMarkerReplication, False),
        "Destination": (ReplicationConfigurationRulesDestination, True),
        "Filter": (ReplicationRuleFilter, False),
        "Id": (str, False),
        "Prefix": (str, False),
        "Priority": (integer, False),
        "SourceSelectionCriteria": (SourceSelectionCriteria, False),
        "Status": (str, True),
    }


class ReplicationConfiguration(AWSProperty):
    props = {"Role": (str, True), "Rules": ([ReplicationConfigurationRules], True)}


class Destination(AWSProperty):
    props = {
        "BucketAccountId": (str, False),
        "BucketArn": (str, True),
        "Format": (str, True),
        "Prefix": (str, False),
    }


class DataExport(AWSProperty):
    props = {
        "Destination": (Destination, True),
        "OutputSchemaVersion": (str, True),
    }


class StorageClassAnalysis(AWSProperty):
    props = {
        "DataExport": (DataExport, False),
    }


class AnalyticsConfiguration(AWSProperty):
    props = {
        "Id": (str, True),
        "Prefix": (str, False),
        "StorageClassAnalysis": (StorageClassAnalysis, True),
        "TagFilters": ([TagFilter], False),
    }


class ServerSideEncryptionByDefault(AWSProperty):
    props = {
        "KMSMasterKeyID": (str, False),
        "SSEAlgorithm": (str, True),
    }


class ServerSideEncryptionRule(AWSProperty):
    props = {
        "BucketKeyEnabled": (boolean, False),
        "ServerSideEncryptionByDefault": (ServerSideEncryptionByDefault, False),
    }


class BucketEncryption(AWSProperty):
    props = {
        "ServerSideEncryptionConfiguration": ([ServerSideEncryptionRule], True),
    }


class InventoryConfiguration(AWSProperty):
    props = {
        "Destination": (Destination, True),
        "Enabled": (boolean, True),
        "Id": (str, True),
        "IncludedObjectVersions": (str, True),
        "OptionalFields": ([str], True),
        "Prefix": (str, False),
        "ScheduleFrequency": (str, True),
    }


class DefaultRetention(AWSProperty):
    props = {
        "Days": (integer, False),
        "Mode": (str, False),
        "Years": (integer, False),
    }


class ObjectLockRule(AWSProperty):
    props = {
        "DefaultRetention": (DefaultRetention, False),
    }


class ObjectLockConfiguration(AWSProperty):
    props = {
        "ObjectLockEnabled": (str, False),
        "Rule": (ObjectLockRule, False),
    }


class Bucket(AWSObject):
    resource_type = "AWS::S3::Bucket"

    props = {
        "AccelerateConfiguration": (AccelerateConfiguration, False),
        "AccessControl": (str, False),
        "AnalyticsConfigurations": ([AnalyticsConfiguration], False),
        "BucketEncryption": (BucketEncryption, False),
        "BucketName": (s3_bucket_name, False),
        "CorsConfiguration": (CorsConfiguration, False),
        "InventoryConfigurations": ([InventoryConfiguration], False),
        "LifecycleConfiguration": (LifecycleConfiguration, False),
        "LoggingConfiguration": (LoggingConfiguration, False),
        "MetricsConfigurations": ([MetricsConfiguration], False),
        "NotificationConfiguration": (NotificationConfiguration, False),
        "ObjectLockConfiguration": (ObjectLockConfiguration, False),
        "ObjectLockEnabled": (boolean, False),
        "OwnershipControls": (OwnershipControls, False),
        "PublicAccessBlockConfiguration": (PublicAccessBlockConfiguration, False),
        "ReplicationConfiguration": (ReplicationConfiguration, False),
        "Tags": (Tags, False),
        "VersioningConfiguration": (VersioningConfiguration, False),
        "WebsiteConfiguration": (WebsiteConfiguration, False),
    }

    access_control_types = [
        Private,
        PublicRead,
        PublicReadWrite,
        AuthenticatedRead,
        BucketOwnerRead,
        BucketOwnerFullControl,
        LogDeliveryWrite,
    ]

    def validate(self):
        access_control = self.properties.get("AccessControl")
        if access_control is not None and not isinstance(access_control, AWSHelperFn):
            if access_control not in self.access_control_types:
                raise ValueError(
                    'AccessControl must be one of "%s"'
                    % (", ".join(self.access_control_types))
                )


class BucketPolicy(AWSObject):
    resource_type = "AWS::S3::BucketPolicy"

    props = {
        "Bucket": (str, True),
        "PolicyDocument": (policytypes, True),
    }


class ActivityMetrics(AWSProperty):
    props = {
        "IsEnabled": (boolean, False),
    }


class SelectionCriteria(AWSProperty):
    props = {
        "Delimiter": (str, False),
        "MaxDepth": (integer, False),
        "MinStorageBytesPercentage": (double, False),
    }


class PrefixLevelStorageMetrics(AWSProperty):
    props = {
        "IsEnabled": (boolean, False),
        "SelectionCriteria": (SelectionCriteria, False),
    }


class PrefixLevel(AWSProperty):
    props = {
        "StorageMetrics": (PrefixLevelStorageMetrics, True),
    }


class BucketLevel(AWSProperty):
    props = {
        "ActivityMetrics": (ActivityMetrics, False),
        "PrefixLevel": (PrefixLevel, False),
    }


class AccountLevel(AWSProperty):
    props = {
        "ActivityMetrics": (ActivityMetrics, False),
        "BucketLevel": (BucketLevel, True),
    }


class AwsOrg(AWSProperty):
    props = {
        "Arn": (str, True),
    }


class BucketsAndRegions(AWSProperty):
    props = {
        "Buckets": ([str], False),
        "Regions": ([str], False),
    }


class StorageLensConfiguration(AWSProperty):
    props = {
        "AccountLevel": (AccountLevel, True),
        "AwsOrg": (AwsOrg, False),
        "DataExport": (DataExport, False),
        "Exclude": (BucketsAndRegions, False),
        "Id": (str, True),
        "Include": (BucketsAndRegions, False),
        "IsEnabled": (boolean, True),
        "StorageLensArn": (str, False),
    }


class StorageLens(AWSObject):
    resource_type = "AWS::S3::StorageLens"

    props = {
        "StorageLensConfiguration": (StorageLensConfiguration, True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class AWSAccount(AWSProperty):
    props = {
        "EmailAddress": (str, False),
        "Id": (str, False),
        "Name": (str, False),
    }


class AssessmentReportsDestination(AWSProperty):
    props = {
        "Destination": (str, False),
        "DestinationType": (str, False),
    }


class Role(AWSProperty):
    props = {
        "RoleArn": (str, False),
        "RoleType": (str, False),
    }


class AWSService(AWSProperty):
    props = {
        "ServiceName": (str, False),
    }


class Scope(AWSProperty):
    props = {
        "AwsAccounts": ([AWSAccount], False),
        "AwsServices": ([AWSService], False),
    }


class Assessment(AWSObject):
    resource_type = "AWS::AuditManager::Assessment"

    props = {
        "AssessmentReportsDestination": (AssessmentReportsDestination, False),
        "AwsAccount": (AWSAccount, False),
        "Description": (str, False),
        "FrameworkId": (str, False),
        "Name": (str, False),
        "Roles": ([Role], False),
        "Scope": (Scope, False),
        "Status": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, integer

VALID_LISTENERTLS_MODE = ("STRICT", "PERMISSIVE", "DISABLED")


def validate_listenertls_mode(listenertls_mode):
    """Validate Mode for ListernerTls"""

    if listenertls_mode not in VALID_LISTENERTLS_MODE:
        raise ValueError(
            "ListernerTls Mode must be one of: %s" % ", ".join(VALID_LISTENERTLS_MODE)
        )
    return listenertls_mode


class GatewayRouteVirtualService(AWSProperty):
    props = {
        "VirtualServiceName": (str, True),
    }


class GatewayRouteTarget(AWSProperty):
    props = {
        "VirtualService": (GatewayRouteVirtualService, True),
    }


class GatewayRouteHostnameRewrite(AWSProperty):
    props = {
        "DefaultTargetHostname": (str, False),
    }


class GrpcGatewayRouteRewrite(AWSProperty):
    props = {
        "Hostname": (GatewayRouteHostnameRewrite, False),
    }


class GrpcGatewayRouteAction(AWSProperty):
    props = {
        "Rewrite": (GrpcGatewayRouteRewrite, False),
        "Target": (GatewayRouteTarget, True),
    }


class GatewayRouteHostnameMatch(AWSProperty):
    props = {
        "Exact": (str, False),
        "Suffix": (str, False),
    }


class GatewayRouteRangeMatch(AWSProperty):
    props = {
        "End": (integer, True),
        "Start": (integer, True),
    }


class GatewayRouteMetadataMatch(AWSProperty):
    props = {
        "Exact": (str, False),
        "Prefix": (str, False),
        "Range": (GatewayRouteRangeMatch, False),
        "Regex": (str, False),
        "Suffix": (str, False),
    }


class GrpcGatewayRouteMetadata(AWSProperty):
    props = {
        "Invert": (boolean, False),
        "Match": (GatewayRouteMetadataMatch, False),
        "Name": (str, True),
    }


class GrpcGatewayRouteMatch(AWSProperty):
    props = {
        "Hostname": (GatewayRouteHostnameMatch, False),
        "Metadata": ([GrpcGatewayRouteMetadata], False),
        "ServiceName": (str, False),
    }


class GrpcGatewayRoute(AWSProperty):
    props = {
        "Action": (GrpcGatewayRouteAction, True),
        "Match": (GrpcGatewayRouteMatch, True),
    }


class HttpGatewayRoutePathRewrite(AWSProperty):
    props = {
        "Exact": (str, False),
    }


class HttpGatewayRoutePrefixRewrite(AWSProperty):
    props = {
        "DefaultPrefix": (str, False),
        "Value": (str, False),
    }


class HttpGatewayRouteRewrite(AWSProperty):
    props = {
        "Hostname": (GatewayRouteHostnameRewrite, False),
        "Path": (HttpGatewayRoutePathRewrite, False),
        "Prefix": (HttpGatewayRoutePrefixRewrite, False),
    }


class HttpGatewayRouteAction(AWSProperty):
    props = {
        "Rewrite": (HttpGatewayRouteRewrite, False),
        "Target": (GatewayRouteTarget, True),
    }


class HttpGatewayRouteHeaderMatch(AWSProperty):
    props = {
        "Exact": (str, False),
        "Prefix": (str, False),
        "Range": (GatewayRouteRangeMatch, False),
        "Regex": (str, False),
        "Suffix": (str, False),
    }


class HttpGatewayRouteHeader(AWSProperty):
    props = {
        "Invert": (boolean, False),
        "Match": (HttpGatewayRouteHeaderMatch, False),
        "Name": (str, True),
    }


class HttpPathMatch(AWSProperty):
    props = {
        "Exact": (str, False),
        "Regex": (str, False),
    }


class HttpQueryParameterMatch(AWSProperty):
    props = {
        "Exact": (str, False),
    }


class QueryParameter(AWSProperty):
    props = {
        "Match": (HttpQueryParameterMatch, False),
        "Name": (str, True),
    }


class HttpGatewayRouteMatch(AWSProperty):
    props = {
        "Headers": ([HttpGatewayRouteHeader], False),
        "Hostname": (GatewayRouteHostnameMatch, False),
        "Method": (str, False),
        "Path": (HttpPathMatch, False),
        "Prefix": (str, False),
        "QueryParameters": ([QueryParameter], False),
    }


class HttpGatewayRoute(AWSProperty):
    props = {
        "Action": (HttpGatewayRouteAction, True),
        "Match": (HttpGatewayRouteMatch, True),
    }


class GatewayRouteSpec(AWSProperty):
    props = {
        "GrpcRoute": (GrpcGatewayRoute, False),
        "Http2Route": (HttpGatewayRoute, False),
        "HttpRoute": (HttpGatewayRoute, False),
    }


class GatewayRoute(AWSObject):
    resource_type = "AWS::AppMesh::GatewayRoute"

    props = {
        "GatewayRouteName": (str, False),
        "MeshName": (str, True),
        "MeshOwner": (str, False),
        "Spec": (GatewayRouteSpec, True),
        "Tags": (Tags, False),
        "VirtualGatewayName": (str, True),
    }


class EgressFilter(AWSProperty):
    props = {
        "Type": (str, True),
    }


class MeshSpec(AWSProperty):
    props = {
        "EgressFilter": (EgressFilter, False),
    }


class Mesh(AWSObject):
    resource_type = "AWS::AppMesh::Mesh"

    props = {
        "MeshName": (str, False),
        "Spec": (MeshSpec, False),
        "Tags": (Tags, False),
    }


class Duration(AWSProperty):
    props = {
        "Unit": (str, True),
        "Value": (integer, True),
    }


class GrpcRetryPolicy(AWSProperty):
    props = {
        "GrpcRetryEvents": ([str], False),
        "HttpRetryEvents": ([str], False),
        "MaxRetries": (integer, True),
        "PerRetryTimeout": (Duration, True),
        "TcpRetryEvents": ([str], False),
    }


class WeightedTarget(AWSProperty):
    props = {
        "VirtualNode": (str, True),
        "Weight": (integer, True),
    }


class GrpcRouteAction(AWSProperty):
    props = {
        "WeightedTargets": ([WeightedTarget], True),
    }


class MatchRange(AWSProperty):
    props = {
        "End": (integer, True),
        "Start": (integer, True),
    }


class GrpcRouteMetadataMatchMethod(AWSProperty):
    props = {
        "Exact": (str, False),
        "Prefix": (str, False),
        "Range": (MatchRange, False),
        "Regex": (str, False),
        "Suffix": (str, False),
    }


class GrpcRouteMetadata(AWSProperty):
    props = {
        "Invert": (boolean, False),
        "Match": (GrpcRouteMetadataMatchMethod, False),
        "Name": (str, True),
    }


class GrpcRouteMatch(AWSProperty):
    props = {
        "Metadata": ([GrpcRouteMetadata], False),
        "MethodName": (str, False),
        "ServiceName": (str, False),
    }


class GrpcTimeout(AWSProperty):
    props = {
        "Idle": (Duration, False),
        "PerRequest": (Duration, False),
    }


class GrpcRoute(AWSProperty):
    props = {
        "Action": (GrpcRouteAction, True),
        "Match": (GrpcRouteMatch, True),
        "RetryPolicy": (GrpcRetryPolicy, False),
        "Timeout": (GrpcTimeout, False),
    }


class HttpRetryPolicy(AWSProperty):
    props = {
        "HttpRetryEvents": ([str], False),
        "MaxRetries": (integer, True),
        "PerRetryTimeout": (Duration, True),
        "TcpRetryEvents": ([str], False),
    }


class HttpRouteAction(AWSProperty):
    props = {
        "WeightedTargets": ([WeightedTarget], True),
    }


class HeaderMatchMethod(AWSProperty):
    props = {
        "Exact": (str, False),
        "Prefix": (str, False),
        "Range": (MatchRange, False),
        "Regex": (str, False),
        "Suffix": (str, False),
    }


class HttpRouteHeader(AWSProperty):
    props = {
        "Invert": (boolean, False),
        "Match": (HeaderMatchMethod, False),
        "Name": (str, True),
    }


class HttpRouteMatch(AWSProperty):
    props = {
        "Headers": ([HttpRouteHeader], False),
        "Method": (str, False),
        "Prefix": (str, True),
        "Scheme": (str, False),
    }


class HttpTimeout(AWSProperty):
    props = {
        "Idle": (Duration, False),
        "PerRequest": (Duration, False),
    }


class HttpRoute(AWSProperty):
    props = {
        "Action": (HttpRouteAction, True),
        "Match": (HttpRouteMatch, True),
        "RetryPolicy": (HttpRetryPolicy, False),
        "Timeout": (HttpTimeout, False),
    }


class TcpRouteAction(AWSProperty):
    props = {
        "WeightedTargets": ([WeightedTarget], True),
    }


class TcpTimeout(AWSProperty):
    props = {
        "Idle": (Duration, False),
    }


class TcpRoute(AWSProperty):
    props = {
        "Action": (TcpRouteAction, True),
        "Timeout": (TcpTimeout, False),
    }


class RouteSpec(AWSProperty):
    props = {
        "GrpcRoute": (GrpcRoute, False),
        "Http2Route": (HttpRoute, False),
        "HttpRoute": (HttpRoute, False),
        "Priority": (integer, False),
        "TcpRoute": (TcpRoute, False),
    }


class Route(AWSObject):
    resource_type = "AWS::AppMesh::Route"

    props = {
        "MeshName": (str, True),
        "MeshOwner": (str, False),
        "RouteName": (str, False),
        "Spec": (RouteSpec, True),
        "Tags": (Tags, False),
        "VirtualRouterName": (str, True),
    }


class VirtualGatewayListenerTlsFileCertificate(AWSProperty):
    props = {
        "CertificateChain": (str, True),
        "PrivateKey": (str, True),
    }


class VirtualGatewayListenerTlsSdsCertificate(AWSProperty):
    props = {
        "SecretName": (str, True),
    }


class VirtualGatewayClientTlsCertificate(AWSProperty):
    props = {
        "File": (VirtualGatewayListenerTlsFileCertificate, False),
        "SDS": (VirtualGatewayListenerTlsSdsCertificate, False),
    }


class SubjectAlternativeNameMatchers(AWSProperty):
    props = {
        "Exact": ([str], False),
    }


class SubjectAlternativeNames(AWSProperty):
    props = {
        "Match": (SubjectAlternativeNameMatchers, True),
    }


class VirtualGatewayTlsValidationContextAcmTrust(AWSProperty):
    props = {
        "CertificateAuthorityArns": ([str], True),
    }


class VirtualGatewayTlsValidationContextFileTrust(AWSProperty):
    props = {
        "CertificateChain": (str, True),
    }


class VirtualGatewayTlsValidationContextSdsTrust(AWSProperty):
    props = {
        "SecretName": (str, True),
    }


class VirtualGatewayTlsValidationContextTrust(AWSProperty):
    props = {
        "ACM": (VirtualGatewayTlsValidationContextAcmTrust, False),
        "File": (VirtualGatewayTlsValidationContextFileTrust, False),
        "SDS": (VirtualGatewayTlsValidationContextSdsTrust, False),
    }


class VirtualGatewayTlsValidationContext(AWSProperty):
    props = {
        "SubjectAlternativeNames": (SubjectAlternativeNames, False),
        "Trust": (VirtualGatewayTlsValidationContextTrust, True),
    }


class VirtualGatewayClientPolicyTls(AWSProperty):
    props = {
        "Certificate": (VirtualGatewayClientTlsCertificate, False),
        "Enforce": (boolean, False),
        "Ports": ([integer], False),
        "Validation": (VirtualGatewayTlsValidationContext, True),
    }


class VirtualGatewayClientPolicy(AWSProperty):
    props = {
        "TLS": (VirtualGatewayClientPolicyTls, False),
    }


class VirtualGatewayBackendDefaults(AWSProperty):
    props = {
        "ClientPolicy": (VirtualGatewayClientPolicy, False),
    }


class VirtualGatewayGrpcConnectionPool(AWSProperty):
    props = {
        "MaxRequests": (integer, True),
    }


class VirtualGatewayHttp2ConnectionPool(AWSProperty):
    props = {
        "MaxRequests": (integer, True),
    }


class VirtualGatewayHttpConnectionPool(AWSProperty):
    props = {
        "MaxConnections": (integer, True),
        "MaxPendingRequests": (integer, False),
    }


class VirtualGatewayConnectionPool(AWSProperty):
    props = {
        "GRPC": (VirtualGatewayGrpcConnectionPool, False),
        "HTTP": (VirtualGatewayHttpConnectionPool, False),
        "HTTP2": (VirtualGatewayHttp2ConnectionPool, False),
    }


class VirtualGatewayHealthCheckPolicy(AWSProperty):
    props = {
        "HealthyThreshold": (integer, True),
        "IntervalMillis": (integer, True),
        "Path": (str, False),
        "Port": (integer, False),
        "Protocol": (str, True),
        "TimeoutMillis": (integer, True),
        "UnhealthyThreshold": (integer, True),
    }


class VirtualGatewayListenerTlsAcmCertificate(AWSProperty):
    props = {
        "CertificateArn": (str, True),
    }


class VirtualGatewayListenerTlsCertificate(AWSProperty):
    props = {
        "ACM": (VirtualGatewayListenerTlsAcmCertificate, False),
        "File": (VirtualGatewayListenerTlsFileCertificate, False),
        "SDS": (VirtualGatewayListenerTlsSdsCertificate, False),
    }


class VirtualGatewayListenerTlsValidationContextTrust(AWSProperty):
    props = {
        "File": (VirtualGatewayTlsValidationContextFileTrust, False),
        "SDS": (VirtualGatewayTlsValidationContextSdsTrust, False),
    }


class VirtualGatewayListenerTlsValidationContext(AWSProperty):
    props = {
        "SubjectAlternativeNames": (SubjectAlternativeNames, False),
        "Trust": (VirtualGatewayListenerTlsValidationContextTrust, True),
    }


class VirtualGatewayListenerTls(AWSProperty):
    props = {
        "Certificate": (VirtualGatewayListenerTlsCertificate, True),
        "Mode": (str, True),
        "Validation": (VirtualGatewayListenerTlsValidationContext, False),
    }


class VirtualGatewayPortMapping(AWSProperty):
    props = {
        "Port": (integer, True),
        "Protocol": (str, True),
    }


class VirtualGatewayListener(AWSProperty):
    props = {
        "ConnectionPool": (VirtualGatewayConnectionPool, False),
        "HealthCheck": (VirtualGatewayHealthCheckPolicy, False),
        "PortMapping": (VirtualGatewayPortMapping, True),
        "TLS": (VirtualGatewayListenerTls, False),
    }


class VirtualGatewayFileAccessLog(AWSProperty):
    props = {
        "Path": (str, True),
    }


class VirtualGatewayAccessLog(AWSProperty):
    props = {
        "File": (VirtualGatewayFileAccessLog, False),
    }


class VirtualGatewayLogging(AWSProperty):
    props = {
        "AccessLog": (VirtualGatewayAccessLog, False),
    }


class VirtualGatewaySpec(AWSProperty):
    props = {
        "BackendDefaults": (VirtualGatewayBackendDefaults, False),
        "Listeners": ([VirtualGatewayListener], True),
        "Logging": (VirtualGatewayLogging, False),
    }


class VirtualGateway(AWSObject):
    resource_type = "AWS::AppMesh::VirtualGateway"

    props = {
        "MeshName": (str, True),
        "MeshOwner": (str, False),
        "Spec": (VirtualGatewaySpec, True),
        "Tags": (Tags, False),
        "VirtualGatewayName": (str, False),
    }


class ListenerTlsFileCertificate(AWSProperty):
    props = {
        "CertificateChain": (str, True),
        "PrivateKey": (str, True),
    }


class ListenerTlsSdsCertificate(AWSProperty):
    props = {
        "SecretName": (str, True),
    }


class ClientTlsCertificate(AWSProperty):
    props = {
        "File": (ListenerTlsFileCertificate, False),
        "SDS": (ListenerTlsSdsCertificate, False),
    }


class TlsValidationContextAcmTrust(AWSProperty):
    props = {
        "CertificateAuthorityArns": ([str], True),
    }


class TlsValidationContextFileTrust(AWSProperty):
    props = {
        "CertificateChain": (str, True),
    }


class TlsValidationContextSdsTrust(AWSProperty):
    props = {
        "SecretName": (str, True),
    }


class TlsValidationContextTrust(AWSProperty):
    props = {
        "ACM": (TlsValidationContextAcmTrust, False),
        "File": (TlsValidationContextFileTrust, False),
        "SDS": (TlsValidationContextSdsTrust, False),
    }


class TlsValidationContext(AWSProperty):
    props = {
        "SubjectAlternativeNames": (SubjectAlternativeNames, False),
        "Trust": (TlsValidationContextTrust, True),
    }


class ClientPolicyTls(AWSProperty):
    props = {
        "Certificate": (ClientTlsCertificate, False),
        "Enforce": (boolean, False),
        "Ports": ([integer], False),
        "Validation": (TlsValidationContext, True),
    }


class ClientPolicy(AWSProperty):
    props = {
        "TLS": (ClientPolicyTls, False),
    }


class VirtualServiceBackend(AWSProperty):
    props = {
        "ClientPolicy": (ClientPolicy, False),
        "VirtualServiceName": (str, True),
    }


class Backend(AWSProperty):
    props = {
        "VirtualService": (VirtualServiceBackend, False),
    }


class BackendDefaults(AWSProperty):
    props = {
        "ClientPolicy": (ClientPolicy, False),
    }


class HealthCheck(AWSProperty):
    props = {
        "HealthyThreshold": (integer, True),
        "IntervalMillis": (integer, True),
        "Path": (str, False),
        "Port": (integer, False),
        "Protocol": (str, True),
        "TimeoutMillis": (integer, True),
        "UnhealthyThreshold": (integer, True),
    }


class ListenerTimeout(AWSProperty):
    props = {
        "GRPC": (GrpcTimeout, False),
        "HTTP": (HttpTimeout, False),
        "HTTP2": (HttpTimeout, False),
        "TCP": (TcpTimeout, False),
    }


class ListenerTlsAcmCertificate(AWSProperty):
    props = {
        "CertificateArn": (str, True),
    }


class ListenerTlsCertificate(AWSProperty):
    props = {
        "ACM": (ListenerTlsAcmCertificate, False),
        "File": (ListenerTlsFileCertificate, False),
        "SDS": (ListenerTlsSdsCertificate, False),
    }


class ListenerTlsValidationContextTrust(AWSProperty):
    props = {
        "File": (TlsValidationContextFileTrust, False),
        "SDS": (TlsValidationContextSdsTrust, False),
    }


class ListenerTlsValidationContext(AWSProperty):
    props = {
        "SubjectAlternativeNames": (SubjectAlternativeNames, False),
        "Trust": (ListenerTlsValidationContextTrust, True),
    }


class ListenerTls(AWSProperty):
    props = {
        "Certificate": (ListenerTlsCertificate, True),
        "Mode": (validate_listenertls_mode, True),
        "Validation": (ListenerTlsValidationContext, False),
    }


class OutlierDetection(AWSProperty):
    props = {
        "BaseEjectionDuration": (Duration, True),
        "Interval": (Duration, True),
        "MaxEjectionPercent": (integer, True),
        "MaxServerErrors": (integer, True),
    }


class PortMapping(AWSProperty):
    props = {
        "Port": (integer, True),
        "Protocol": (str, True),
    }


class VirtualNodeGrpcConnectionPool(AWSProperty):
    props = {
        "MaxRequests": (integer, True),
    }


class VirtualNodeHttp2ConnectionPool(AWSProperty):
    props = {
        "MaxRequests": (integer, True),
    }


class VirtualNodeHttpConnectionPool(AWSProperty):
    props = {
        "MaxConnections": (integer, True),
        "MaxPendingRequests": (integer, False),
    }


class VirtualNodeTcpConnectionPool(AWSProperty):
    props = {
        "MaxConnections": (integer, True),
    }


class VirtualNodeConnectionPool(AWSProperty):
    props = {
        "GRPC": (VirtualNodeGrpcConnectionPool, False),
        "HTTP": (VirtualNodeHttpConnectionPool, False),
        "HTTP2": (VirtualNodeHttp2ConnectionPool, False),
        "TCP": (VirtualNodeTcpConnectionPool, False),
    }


class Listener(AWSProperty):
    props = {
        "ConnectionPool": (VirtualNodeConnectionPool, False),
        "HealthCheck": (HealthCheck, False),
        "OutlierDetection": (OutlierDetection, False),
        "PortMapping": (PortMapping, True),
        "TLS": (ListenerTls, False),
        "Timeout": (ListenerTimeout, False),
    }


class FileAccessLog(AWSProperty):
    props = {
        "Path": (str, True),
    }


class AccessLog(AWSProperty):
    props = {
        "File": (FileAccessLog, False),
    }


class Logging(AWSProperty):
    props = {
        "AccessLog": (AccessLog, False),
    }


class AwsCloudMapInstanceAttribute(AWSProperty):
    props = {
        "Key": (str, True),
        "Value": (str, True),
    }


class AwsCloudMapServiceDiscovery(AWSProperty):
    props = {
        "Attributes": ([AwsCloudMapInstanceAttribute], False),
        "NamespaceName": (str, True),
        "ServiceName": (str, True),
    }


class DnsServiceDiscovery(AWSProperty):
    props = {
        "Hostname": (str, True),
    }


class ServiceDiscovery(AWSProperty):
    props = {
        "AWSCloudMap": (AwsCloudMapServiceDiscovery, False),
        "DNS": (DnsServiceDiscovery, False),
    }


class VirtualNodeSpec(AWSProperty):
    props = {
        "BackendDefaults": (BackendDefaults, False),
        "Backends": ([Backend], False),
        "Listeners": ([Listener], False),
        "Logging": (Logging, False),
        "ServiceDiscovery": (ServiceDiscovery, False),
    }


class VirtualNode(AWSObject):
    resource_type = "AWS::AppMesh::VirtualNode"

    props = {
        "MeshName": (str, True),
        "MeshOwner": (str, False),
        "Spec": (VirtualNodeSpec, True),
        "Tags": (Tags, False),
        "VirtualNodeName": (str, False),
    }


class VirtualRouterListener(AWSProperty):
    props = {
        "PortMapping": (PortMapping, True),
    }


class VirtualRouterSpec(AWSProperty):
    props = {
        "Listeners": ([VirtualRouterListener], True),
    }


class VirtualRouter(AWSObject):
    resource_type = "AWS::AppMesh::VirtualRouter"

    props = {
        "MeshName": (str, True),
        "MeshOwner": (str, False),
        "Spec": (VirtualRouterSpec, True),
        "Tags": (Tags, False),
        "VirtualRouterName": (str, False),
    }


class VirtualNodeServiceProvider(AWSProperty):
    props = {
        "VirtualNodeName": (str, True),
    }


class VirtualRouterServiceProvider(AWSProperty):
    props = {
        "VirtualRouterName": (str, True),
    }


class VirtualServiceProvider(AWSProperty):
    props = {
        "VirtualNode": (VirtualNodeServiceProvider, False),
        "VirtualRouter": (VirtualRouterServiceProvider, False),
    }


class VirtualServiceSpec(AWSProperty):
    props = {
        "Provider": (VirtualServiceProvider, False),
    }


class VirtualService(AWSObject):
    resource_type = "AWS::AppMesh::VirtualService"

    props = {
        "MeshName": (str, True),
        "MeshOwner": (str, False),
        "Spec": (VirtualServiceSpec, True),
        "Tags": (Tags, False),
        "VirtualServiceName": (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#

from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import integer


class AudioLanguageSelection(AWSProperty):
    props = {
        "LanguageCode": (str, False),
        "LanguageSelectionPolicy": (str, False),
    }


class AudioPidSelection(AWSProperty):
    props = {
        "Pid": (integer, False),
    }


class AudioSelectorSettings(AWSProperty):
    props = {
        "AudioLanguageSelection": (AudioLanguageSelection, False),
        "AudioPidSelection": (AudioPidSelection, False),
    }


class AudioSelector(AWSProperty):
    props = {
        "Name": (str, False),
        "SelectorSettings": (AudioSelectorSettings, False),
    }


class AribSourceSettings(AWSProperty):
    props = {}


class DvbSubSourceSettings(AWSProperty):
    props = {
        "Pid": (integer, False),
    }


class EmbeddedSourceSettings(AWSProperty):
    props = {
        "Convert608To708": (str, False),
        "Scte20Detection": (str, False),
        "Source608ChannelNumber": (integer, False),
        "Source608TrackNumber": (integer, False),
    }


class Scte20SourceSettings(AWSProperty):
    props = {
        "Convert608To708": (str, False),
        "Source608ChannelNumber": (integer, False),
    }


class Scte27SourceSettings(AWSProperty):
    props = {
        "Pid": (integer, False),
    }


class TeletextSourceSettings(AWSProperty):
    props = {
        "PageNumber": (str, False),
    }


class CaptionSelectorSettings(AWSProperty):
    props = {
        "AribSourceSettings": (AribSourceSettings, False),
        "DvbSubSourceSettings": (DvbSubSourceSettings, False),
        "EmbeddedSourceSettings": (EmbeddedSourceSettings, False),
        "Scte20SourceSettings": (Scte20SourceSettings, False),
        "Scte27SourceSettings": (Scte27SourceSettings, False),
        "TeletextSourceSettings": (TeletextSourceSettings, False),
    }


class CaptionSelector(AWSProperty):
    props = {
        "LanguageCode": (str, False),
        "Name": (str, False),
        "SelectorSettings": (CaptionSelectorSettings, False),
    }


class HlsInputSettings(AWSProperty):
    props = {
        "Bandwidth": (integer, False),
        "BufferSegments": (integer, False),
        "Retries": (integer, False),
        "RetryInterval": (integer, False),
    }


class NetworkInputSettings(AWSProperty):
    props = {
        "HlsInputSettings": (HlsInputSettings, False),
        "ServerValidation": (str, False),
    }


class VideoSelectorPid(AWSProperty):
    props = {
        "Pid": (integer, False),
    }


class VideoSelectorProgramId(AWSProperty):
    props = {
        "ProgramId": (integer, False),
    }


class VideoSelectorSettings(AWSProperty):
    props = {
        "VideoSelectorPid": (VideoSelectorPid, False),
        "VideoSelectorProgramId": (VideoSelectorProgramId, False),
    }


class VideoSelector(AWSProperty):
    props = {
        "ColorSpace": (str, False),
        "ColorSpaceUsage": (str, False),
        "SelectorSettings": (VideoSelectorSettings, False),
    }


class InputSettings(AWSProperty):
    props = {
        "AudioSelectors": ([AudioSelector], False),
        "CaptionSelectors": ([CaptionSelector], False),
        "DeblockFilter": (str, False),
        "DenoiseFilter": (str, False),
        "FilterStrength": (integer, False),
        "InputFilter": (str, False),
        "NetworkInputSettings": (NetworkInputSettings, False),
        "SourceEndBehavior": (str, False),
        "VideoSelector": (VideoSelector, False),
    }


class InputAttachment(AWSProperty):
    props = {
        "InputAttachmentName": (str, False),
        "InputId": (str, False),
        "InputSettings": (InputSettings, False),
    }


class InputSpecification(AWSProperty):
    props = {
        "Codec": (str, False),
        "MaximumBitrate": (str, False),
        "Resolution": (str, False),
    }


class MediaPackageOutputDestinationSettings(AWSProperty):
    props = {
        "ChannelId": (str, False),
    }


class OutputDestinationSettings(AWSProperty):
    props = {
        "PasswordParam": (str, False),
        "StreamName": (str, False),
        "Url": (str, False),
        "Username": (str, False),
    }


class OutputDestination(AWSProperty):
    props = {
        "Id": (str, False),
        "MediaPackageSettings": ([MediaPackageOutputDestinationSettings], False),
        "Settings": ([OutputDestinationSettings], False),
    }


class Channel(AWSObject):
    resource_type = "AWS::MediaLive::Channel"

    props = {
        "ChannelClass": (str, False),
        "Destinations": ([OutputDestination], False),
        "EncoderSettings": (dict, False),
        "InputAttachments": ([InputAttachment], False),
        "InputSpecification": (InputSpecification, False),
        "LogLevel": (str, False),
        "Name": (str, False),
        "RoleArn": (str, False),
        "Tags": (Tags, False),
    }


class InputDestinationRequest(AWSProperty):
    props = {
        "StreamName": (str, False),
    }


class InputSourceRequest(AWSProperty):
    props = {
        "PasswordParam": (str, False),
        "Url": (str, False),
        "Username": (str, False),
    }


class InputVpcRequest(AWSProperty):
    props = {
        "SecurityGroupIds": ([str], False),
        "SubnetIds": ([str], False),
    }


class MediaConnectFlowRequest(AWSProperty):
    props = {
        "FlowArn": (str, False),
    }


class Input(AWSObject):
    resource_type = "AWS::MediaLive::Input"

    props = {
        "Destinations": ([InputDestinationRequest], False),
        "InputSecurityGroups": ([str], False),
        "MediaConnectFlows": ([MediaConnectFlowRequest], False),
        "Name": (str, False),
        "RoleArn": (str, False),
        "Sources": ([InputSourceRequest], False),
        "Tags": (Tags, False),
        "Type": (str, False),
        "Vpc": (InputVpcRequest, False),
    }


class InputWhitelistRuleCidr(AWSProperty):
    props = {
        "Cidr": (str, False),
    }


class InputSecurityGroup(AWSObject):
    resource_type = "AWS::MediaLive::InputSecurityGroup"

    props = {
        "Tags": (Tags, False),
        "WhitelistRules": ([InputWhitelistRuleCidr], False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.1.0


from . import AWSObject, AWSProperty
from .validators import boolean


class TransformationConfiguration(AWSProperty):
    props = {
        "Actions": ([str], False),
        "ContentTransformation": (dict, False),
    }


class ObjectLambdaConfiguration(AWSProperty):
    props = {
        "AllowedFeatures": ([str], False),
        "CloudWatchMetricsEnabled": (boolean, False),
        "SupportingAccessPoint": (str, True),
        "TransformationConfigurations": ([TransformationConfiguration], True),
    }


class AccessPoint(AWSObject):
    resource_type = "AWS::S3ObjectLambda::AccessPoint"

    props = {
        "Name": (str, True),
        "ObjectLambdaConfiguration": (ObjectLambdaConfiguration, False),
    }


class AccessPointPolicy(AWSObject):
    resource_type = "AWS::S3ObjectLambda::AccessPointPolicy"

    props = {
        "ObjectLambdaAccessPoint": (str, True),
        "PolicyDocument": (dict, True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import integer


class RedrivePolicy(AWSProperty):
    props = {
        "deadLetterTargetArn": (str, False),
        "maxReceiveCount": (integer, False),
    }


class Queue(AWSObject):
    resource_type = "AWS::SQS::Queue"

    props = {
        "ContentBasedDeduplication": (bool, False),
        "DeduplicationScope": (str, False),
        "DelaySeconds": (integer, False),
        "FifoQueue": (bool, False),
        "FifoThroughputLimit": (str, False),
        "KmsMasterKeyId": (str, False),
        "KmsDataKeyReusePeriodSeconds": (integer, False),
        "MaximumMessageSize": (integer, False),
        "MessageRetentionPeriod": (integer, False),
        "QueueName": (str, False),
        "ReceiveMessageWaitTimeSeconds": (integer, False),
        "RedrivePolicy": (RedrivePolicy, False),
        "Tags": (Tags, False),
        "VisibilityTimeout": (integer, False),
    }

    def validate(self):
        if self.properties.get("FifoQueue"):
            queuename = self.properties.get("QueueName")
            if queuename is None or isinstance(queuename, AWSHelperFn):
                pass
            elif not queuename.endswith(".fifo"):
                raise ValueError(
                    "SQS: FIFO queues need to provide a "
                    "QueueName that ends with '.fifo'"
                )


class QueuePolicy(AWSObject):
    resource_type = "AWS::SQS::QueuePolicy"

    props = {
        "PolicyDocument": (policytypes, False),
        "Queues": (list, True),
    }



# Copyright (c) 2016-2017, troposphere project
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer, positive_integer


def processor_type_validator(x):
    valid_types = ["Lambda"]
    if x not in valid_types:
        raise ValueError("Type must be one of: %s" % ", ".join(valid_types))
    return x


def delivery_stream_type_validator(x):
    valid_types = ["DirectPut", "KinesisStreamAsSource"]
    if x not in valid_types:
        raise ValueError(
            "DeliveryStreamType must be one of: %s" % ", ".join(valid_types)
        )
    return x


def index_rotation_period_validator(x):
    valid_types = ["NoRotation", "OneHour", "OneDay", "OneWeek", "OneMonth"]
    if x not in valid_types:
        raise ValueError(
            "IndexRotationPeriod must be one of: %s" % ", ".join(valid_types)
        )
    return x


def s3_backup_mode_elastic_search_validator(x):
    valid_types = ["FailedDocumentsOnly", "AllDocuments"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" % ", ".join(valid_types))
    return x


def s3_backup_mode_extended_s3_validator(x):
    valid_types = ["Disabled", "Enabled"]
    if x not in valid_types:
        raise ValueError("S3BackupMode must be one of: %s" % ", ".join(valid_types))
    return x


class BufferingHints(AWSProperty):
    props = {
        "IntervalInSeconds": (positive_integer, True),
        "SizeInMBs": (positive_integer, True),
    }


class DeliveryStreamEncryptionConfigurationInput(AWSProperty):
    props = {
        "KeyARN": (str, False),
        "KeyType": (str, True),
    }


class CloudWatchLoggingOptions(AWSProperty):
    props = {
        "Enabled": (boolean, False),
        "LogGroupName": (str, False),  # Conditional
        "LogStreamName": (str, False),  # Conditional
    }


class RetryOptions(AWSProperty):
    props = {
        "DurationInSeconds": (positive_integer, True),
    }


class KMSEncryptionConfig(AWSProperty):
    props = {
        "AWSKMSKeyARN": (str, True),
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        "KMSEncryptionConfig": (KMSEncryptionConfig, False),
        "NoEncryptionConfig": (str, False),
    }


class S3Configuration(AWSProperty):
    props = {
        "BucketARN": (str, True),
        "BufferingHints": (BufferingHints, True),
        "CloudWatchLoggingOptions": (CloudWatchLoggingOptions, False),
        "CompressionFormat": (str, True),
        "EncryptionConfiguration": (EncryptionConfiguration, False),
        "Prefix": (str, False),
        "RoleARN": (str, True),
    }


class CopyCommand(AWSProperty):
    props = {
        "CopyOptions": (str, False),
        "DataTableColumns": (str, False),
        "DataTableName": (str, True),
    }


class ProcessorParameter(AWSProperty):
    props = {
        "ParameterName": (str, True),
        "ParameterValue": (str, True),
    }


class Processor(AWSProperty):
    props = {
        "Parameters": ([ProcessorParameter], True),
        "Type": (processor_type_validator, True),
    }


class ProcessingConfiguration(AWSProperty):
    props = {
        "Enabled": (boolean, True),
        "Processors": ([Processor], True),
    }


class VpcConfiguration(AWSProperty):
    props = {
        "RoleARN": (str, True),
        "SecurityGroupIds": ([str], True),
        "SubnetIds": ([str], True),
    }


class ElasticsearchDestinationConfiguration(AWSProperty):
    props = {
        "BufferingHints": (BufferingHints, True),
        "CloudWatchLoggingOptions": (CloudWatchLoggingOptions, False),
        "ClusterEndpoint": (str, False),
        "DomainARN": (str, True),
        "IndexName": (str, True),
        "IndexRotationPeriod": (index_rotation_period_validator, True),
        "ProcessingConfiguration": (ProcessingConfiguration, False),
        "RetryOptions": (RetryOptions, False),
        "RoleARN": (str, True),
        "S3BackupMode": (s3_backup_mode_elastic_search_validator, True),
        "S3Configuration": (S3Configuration, False),
        "TypeName": (str, False),
        "VpcConfiguration": (VpcConfiguration, False),
    }


class RedshiftDestinationConfiguration(AWSProperty):
    props = {
        "CloudWatchLoggingOptions": (CloudWatchLoggingOptions, False),
        "ClusterJDBCURL": (str, True),
        "CopyCommand": (CopyCommand, True),
        "Password": (str, True),
        "ProcessingConfiguration": (ProcessingConfiguration, False),
        "RoleARN": (str, True),
        "S3Configuration": (S3Configuration, True),
        "Username": (str, True),
    }


class S3DestinationConfiguration(AWSProperty):
    props = {
        "BucketARN": (str, True),
        "BufferingHints": (BufferingHints, False),
        "CloudWatchLoggingOptions": (CloudWatchLoggingOptions, False),
        "CompressionFormat": (str, False),
        "EncryptionConfiguration": (EncryptionConfiguration, False),
        "ErrorOutputPrefix": (str, False),
        "Prefix": (str, False),
        "RoleARN": (str, True),
    }


class HiveJsonSerDe(AWSProperty):
    props = {
        "TimestampFormats": ([str], False),
    }


class OpenXJsonSerDe(AWSProperty):
    props = {
        "CaseInsensitive": (boolean, False),
        "ColumnToJsonKeyMappings": (dict, False),
        "ConvertDotsInJsonKeysToUnderscores": (boolean, False),
    }


class Deserializer(AWSProperty):
    props = {
        "HiveJsonSerDe": (HiveJsonSerDe, False),
        "OpenXJsonSerDe": (OpenXJsonSerDe, False),
    }


class InputFormatConfiguration(AWSProperty):
    props = {
        "Deserializer": (Deserializer, True),
    }


class OrcSerDe(AWSProperty):
    props = {
        "BlockSizeBytes": (integer, False),
        "BloomFilterColumns": ([str], False),
        "BloomFilterFalsePositiveProbability": (float, False),
        "Compression": (str, False),
        "DictionaryKeyThreshold": (float, False),
        "EnablePadding": (boolean, False),
        "FormatVersion": (str, False),
        "PaddingTolerance": (float, False),
        "RowIndexStride": (integer, False),
        "StripeSizeBytes": (integer, False),
    }


class ParquetSerDe(AWSProperty):
    props = {
        "BlockSizeBytes": (integer, False),
        "Compression": (str, False),
        "EnableDictionaryCompression": (boolean, False),
        "MaxPaddingBytes": (integer, False),
        "PageSizeBytes": (integer, False),
        "WriterVersion": (str, False),
    }


class Serializer(AWSProperty):
    props = {
        "OrcSerDe": (OrcSerDe, False),
        "ParquetSerDe": (ParquetSerDe, False),
    }


class OutputFormatConfiguration(AWSProperty):
    props = {
        "Serializer": (Serializer, True),
    }


class SchemaConfiguration(AWSProperty):
    props = {
        "CatalogId": (str, True),
        "DatabaseName": (str, True),
        "Region": (str, True),
        "RoleARN": (str, True),
        "TableName": (str, True),
        "VersionId": (str, True),
    }


class DataFormatConversionConfiguration(AWSProperty):
    props = {
        "Enabled": (boolean, True),
        "InputFormatConfiguration": (InputFormatConfiguration, True),
        "OutputFormatConfiguration": (OutputFormatConfiguration, True),
        "SchemaConfiguration": (SchemaConfiguration, True),
    }


class ExtendedS3DestinationConfiguration(AWSProperty):
    props = {
        "BucketARN": (str, True),
        "BufferingHints": (BufferingHints, False),
        "CloudWatchLoggingOptions": (CloudWatchLoggingOptions, False),
        "CompressionFormat": (str, False),
        "DataFormatConversionConfiguration": (DataFormatConversionConfiguration, False),
        "EncryptionConfiguration": (EncryptionConfiguration, False),
        "ErrorOutputPrefix": (str, False),
        "Prefix": (str, False),
        "ProcessingConfiguration": (ProcessingConfiguration, False),
        "RoleARN": (str, True),
        "S3BackupConfiguration": (S3DestinationConfiguration, False),
        "S3BackupMode": (s3_backup_mode_extended_s3_validator, False),
    }


class KinesisStreamSourceConfiguration(AWSProperty):
    props = {"KinesisStreamARN": (str, True), "RoleARN": (str, True)}


class SplunkRetryOptions(AWSProperty):
    props = {
        "DurationInSeconds": (positive_integer, True),
    }


class SplunkDestinationConfiguration(AWSProperty):
    props = {
        "CloudWatchLoggingOptions": (CloudWatchLoggingOptions, False),
        "HECAcknowledgmentTimeoutInSeconds": (positive_integer, False),
        "HECEndpoint": (str, True),
        "HECEndpointType": (str, True),
        "HECToken": (str, True),
        "ProcessingConfiguration": (ProcessingConfiguration, False),
        "RetryOptions": (SplunkRetryOptions, False),
        "S3BackupMode": (str, False),
        "S3Configuration": (S3DestinationConfiguration, True),
    }


class DeliveryStream(AWSObject):
    resource_type = "AWS::KinesisFirehose::DeliveryStream"

    props = {
        "DeliveryStreamEncryptionConfigurationInput": (
            DeliveryStreamEncryptionConfigurationInput,
            False,
        ),
        "DeliveryStreamName": (str, False),
        "DeliveryStreamType": (delivery_stream_type_validator, False),
        "ElasticsearchDestinationConfiguration": (
            ElasticsearchDestinationConfiguration,
            False,
        ),  # noqa
        "ExtendedS3DestinationConfiguration": (
            ExtendedS3DestinationConfiguration,
            False,
        ),  # noqa
        "KinesisStreamSourceConfiguration": (
            KinesisStreamSourceConfiguration,
            False,
        ),  # noqa
        "RedshiftDestinationConfiguration": (
            RedshiftDestinationConfiguration,
            False,
        ),  # noqa
        "S3DestinationConfiguration": (S3DestinationConfiguration, False),
        "SplunkDestinationConfiguration": (SplunkDestinationConfiguration, False),
        "Tags": (Tags, False),
    }



from troposphere import Base64, Join, Ref, Template
from troposphere.autoscaling import AutoScalingGroup, LaunchConfiguration, Metadata
from troposphere.cloudformation import (
    Init,
    InitConfig,
    InitFile,
    InitFiles,
    InitService,
    InitServices,
)
from troposphere.ecs import Cluster
from troposphere.iam import InstanceProfile, PolicyType, Role

t = Template()
t.set_version("2010-09-09")

PolicyEcr = t.add_resource(
    PolicyType(
        "PolicyEcr",
        PolicyName="EcrPolicy",
        PolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Action": ["ecr:GetAuthorizationToken"],
                    "Resource": ["*"],
                    "Effect": "Allow",
                },
                {
                    "Action": [
                        "ecr:GetDownloadUrlForLayer",
                        "ecr:BatchGetImage",
                        "ecr:BatchCheckLayerAvailability",
                    ],
                    "Resource": ["*"],
                    "Effect": "Allow",
                    "Sid": "AllowPull",
                },
            ],
        },
        Roles=[Ref("EcsClusterRole")],
    )
)

PolicyEcs = t.add_resource(
    PolicyType(
        "PolicyEcs",
        PolicyName="EcsPolicy",
        PolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Action": [
                        "ecs:CreateCluster",
                        "ecs:RegisterContainerInstance",
                        "ecs:DeregisterContainerInstance",
                        "ecs:DiscoverPollEndpoint",
                        "ecs:Submit*",
                        "ecs:Poll",
                        "ecs:StartTelemetrySession",
                    ],
                    "Resource": "*",
                    "Effect": "Allow",
                }
            ],
        },
        Roles=[Ref("EcsClusterRole")],
    )
)

PolicyCloudwatch = t.add_resource(
    PolicyType(
        "PolicyCloudwatch",
        PolicyName="Cloudwatch",
        PolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {"Action": ["cloudwatch:*"], "Resource": "*", "Effect": "Allow"}
            ],
        },
        Roles=[Ref("EcsClusterRole")],
    )
)


ContainerInstances = t.add_resource(
    LaunchConfiguration(
        "ContainerInstances",
        Metadata=Metadata(
            Init(
                {
                    "config": InitConfig(
                        files=InitFiles(
                            {
                                "/etc/cfn/cfn-hup.conf": InitFile(
                                    content=Join(
                                        "",
                                        [
                                            "[main]\n",
                                            "stack=",
                                            Ref("AWS::StackId"),  # NOQA
                                            "\n",
                                            "region=",
                                            Ref("AWS::Region"),
                                            "\n",
                                        ],
                                    ),  # NOQA
                                    mode="000400",
                                    owner="root",
                                    group="root",
                                ),
                                "/etc/cfn/hooks.d/cfn-auto-reloader.conf": InitFile(
                                    content=Join(
                                        "",
                                        [
                                            "[cfn-auto-reloader-hook]\n",
                                            "triggers=post.update\n",
                                            "path=Resources.ContainerInstances.Metadata.AWS::CloudFormation::Init\n",  # NOQA
                                            "action=/opt/aws/bin/cfn-init -v ",
                                            "--stack ",
                                            Ref("AWS::StackName"),  # NOQA
                                            " --resource ContainerInstances ",
                                            " --region ",
                                            Ref("AWS::Region"),
                                            "\n",  # NOQA
                                            "runas=root\n",
                                        ],
                                    ),
                                    mode="000400",
                                    owner="root",
                                    group="root",
                                ),
                            },
                        ),
                        services=InitServices(
                            {
                                "cfn-hup": InitService(
                                    ensureRunning="true",
                                    enabled="true",
                                    files=[
                                        "/etc/cfn/cfn-hup.conf",
                                        "/etc/cfn/hooks.d/cfn-auto-reloader.conf",
                                    ],
                                )
                            }
                        ),
                        commands={
                            "01_add_instance_to_cluster": {
                                "command": Join(
                                    "",
                                    [
                                        "#!/bin/bash\n",  # NOQA
                                        "echo ECS_CLUSTER=",  # NOQA
                                        Ref("ECSCluster"),  # NOQA
                                        " >> /etc/ecs/ecs.config",
                                    ],
                                )
                            },  # NOQA
                            "02_install_ssm_agent": {
                                "command": Join(
                                    "",
                                    [
                                        "#!/bin/bash\n",
                                        "yum -y update\n",  # NOQA
                                        "curl https://amazon-ssm-eu-west-1.s3.amazonaws.com/latest/linux_amd64/amazon-ssm-agent.rpm -o amazon-ssm-agent.rpm\n",  # NOQA
                                        "yum install -y amazon-ssm-agent.rpm",  # NOQA
                                    ],
                                )
                            },
                        },
                    )
                }
            ),
        ),
        UserData=Base64(
            Join(
                "",
                [
                    "#!/bin/bash -xe\n",
                    "yum install -y aws-cfn-bootstrap\n",
                    "/opt/aws/bin/cfn-init -v ",
                    "         --stack ",
                    Ref("AWS::StackName"),
                    "         --resource ContainerInstances ",
                    "         --region ",
                    Ref("AWS::Region"),
                    "\n",
                    "/opt/aws/bin/cfn-signal -e $? ",
                    "         --stack ",
                    Ref("AWS::StackName"),
                    "         --resource ECSAutoScalingGroup ",
                    "         --region ",
                    Ref("AWS::Region"),
                    "\n",
                ],
            )
        ),
        ImageId="ami-13f84d60",
        KeyName="yourkey",
        SecurityGroups=["sg-96114ef2"],
        IamInstanceProfile=Ref("EC2InstanceProfile"),
        InstanceType="t2.micro",
        AssociatePublicIpAddress="true",
    )
)


ECSCluster = t.add_resource(
    Cluster(
        "ECSCluster",
    )
)


ECSAutoScalingGroup = t.add_resource(
    AutoScalingGroup(
        "ECSAutoScalingGroup",
        DesiredCapacity="1",
        MinSize="1",
        MaxSize="1",
        VPCZoneIdentifier=["subnet-72849a0a", "subnet-72849a08"],
        AvailabilityZones=["eu-west-1a", "eu-west-1b"],
        LaunchConfigurationName=Ref("ContainerInstances"),
    )
)

EC2InstanceProfile = t.add_resource(
    InstanceProfile(
        "EC2InstanceProfile",
        Path="/",
        Roles=[Ref("EcsClusterRole")],
    )
)

EcsClusterRole = t.add_resource(
    Role(
        "EcsClusterRole",
        Path="/",
        ManagedPolicyArns=["arn:aws:iam::aws:policy/service-role/AmazonEC2RoleforSSM"],
        AssumeRolePolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Action": "sts:AssumeRole",
                    "Principal": {"Service": "ec2.amazonaws.com"},
                    "Effect": "Allow",
                }
            ],
        },
    )
)

print(t.to_json())



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 35.1.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class Destination(AWSObject):
    resource_type = "AWS::IoTWireless::Destination"

    props = {
        "Description": (str, False),
        "Expression": (str, True),
        "ExpressionType": (str, True),
        "Name": (str, True),
        "RoleArn": (str, True),
        "Tags": (Tags, False),
    }


class LoRaWANDeviceProfile(AWSProperty):
    props = {
        "ClassBTimeout": (integer, False),
        "ClassCTimeout": (integer, False),
        "MacVersion": (str, False),
        "MaxDutyCycle": (integer, False),
        "MaxEirp": (integer, False),
        "PingSlotDr": (integer, False),
        "PingSlotFreq": (integer, False),
        "PingSlotPeriod": (integer, False),
        "RegParamsRevision": (str, False),
        "RfRegion": (str, False),
        "Supports32BitFCnt": (boolean, False),
        "SupportsClassB": (boolean, False),
        "SupportsClassC": (boolean, False),
        "SupportsJoin": (boolean, False),
    }


class DeviceProfile(AWSObject):
    resource_type = "AWS::IoTWireless::DeviceProfile"

    props = {
        "LoRaWAN": (LoRaWANDeviceProfile, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class SidewalkAccountInfo(AWSProperty):
    props = {
        "AppServerPrivateKey": (str, True),
    }


class SidewalkUpdateAccount(AWSProperty):
    props = {
        "AppServerPrivateKey": (str, False),
    }


class PartnerAccount(AWSObject):
    resource_type = "AWS::IoTWireless::PartnerAccount"

    props = {
        "AccountLinked": (boolean, False),
        "Fingerprint": (str, False),
        "PartnerAccountId": (str, False),
        "PartnerType": (str, False),
        "Sidewalk": (SidewalkAccountInfo, False),
        "SidewalkUpdate": (SidewalkUpdateAccount, False),
        "Tags": (Tags, False),
    }


class LoRaWANServiceProfile(AWSProperty):
    props = {
        "AddGwMetadata": (boolean, False),
        "ChannelMask": (str, False),
        "DevStatusReqFreq": (integer, False),
        "DlBucketSize": (integer, False),
        "DlRate": (integer, False),
        "DlRatePolicy": (str, False),
        "DrMax": (integer, False),
        "DrMin": (integer, False),
        "HrAllowed": (boolean, False),
        "MinGwDiversity": (integer, False),
        "NwkGeoLoc": (boolean, False),
        "PrAllowed": (boolean, False),
        "RaAllowed": (boolean, False),
        "ReportDevStatusBattery": (boolean, False),
        "ReportDevStatusMargin": (boolean, False),
        "TargetPer": (integer, False),
        "UlBucketSize": (integer, False),
        "UlRate": (integer, False),
        "UlRatePolicy": (str, False),
    }


class ServiceProfile(AWSObject):
    resource_type = "AWS::IoTWireless::ServiceProfile"

    props = {
        "LoRaWAN": (LoRaWANServiceProfile, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class LoRaWANGatewayVersion(AWSProperty):
    props = {
        "Model": (str, False),
        "PackageVersion": (str, False),
        "Station": (str, False),
    }


class LoRaWANUpdateGatewayTaskEntry(AWSProperty):
    props = {
        "CurrentVersion": (LoRaWANGatewayVersion, False),
        "UpdateVersion": (LoRaWANGatewayVersion, False),
    }


class LoRaWANUpdateGatewayTaskCreate(AWSProperty):
    props = {
        "CurrentVersion": (LoRaWANGatewayVersion, False),
        "SigKeyCrc": (integer, False),
        "UpdateSignature": (str, False),
        "UpdateVersion": (LoRaWANGatewayVersion, False),
    }


class UpdateWirelessGatewayTaskCreate(AWSProperty):
    props = {
        "LoRaWAN": (LoRaWANUpdateGatewayTaskCreate, False),
        "UpdateDataRole": (str, False),
        "UpdateDataSource": (str, False),
    }


class TaskDefinition(AWSObject):
    resource_type = "AWS::IoTWireless::TaskDefinition"

    props = {
        "AutoCreateTasks": (boolean, True),
        "LoRaWANUpdateGatewayTaskEntry": (LoRaWANUpdateGatewayTaskEntry, False),
        "Name": (str, False),
        "Tags": (Tags, False),
        "TaskDefinitionType": (str, False),
        "Update": (UpdateWirelessGatewayTaskCreate, False),
    }


class SessionKeysAbpV10x(AWSProperty):
    props = {
        "AppSKey": (str, True),
        "NwkSKey": (str, True),
    }


class AbpV10x(AWSProperty):
    props = {
        "DevAddr": (str, True),
        "SessionKeys": (SessionKeysAbpV10x, True),
    }


class SessionKeysAbpV11(AWSProperty):
    props = {
        "AppSKey": (str, True),
        "FNwkSIntKey": (str, True),
        "NwkSEncKey": (str, True),
        "SNwkSIntKey": (str, True),
    }


class AbpV11(AWSProperty):
    props = {
        "DevAddr": (str, True),
        "SessionKeys": (SessionKeysAbpV11, True),
    }


class OtaaV10x(AWSProperty):
    props = {
        "AppEui": (str, True),
        "AppKey": (str, True),
    }


class OtaaV11(AWSProperty):
    props = {
        "AppKey": (str, True),
        "JoinEui": (str, True),
        "NwkKey": (str, True),
    }


class LoRaWANDevice(AWSProperty):
    props = {
        "AbpV10x": (AbpV10x, False),
        "AbpV11": (AbpV11, False),
        "DevEui": (str, False),
        "DeviceProfileId": (str, False),
        "OtaaV10x": (OtaaV10x, False),
        "OtaaV11": (OtaaV11, False),
        "ServiceProfileId": (str, False),
    }


class WirelessDevice(AWSObject):
    resource_type = "AWS::IoTWireless::WirelessDevice"

    props = {
        "Description": (str, False),
        "DestinationName": (str, True),
        "LastUplinkReceivedAt": (str, False),
        "LoRaWAN": (LoRaWANDevice, False),
        "Name": (str, False),
        "Tags": (Tags, False),
        "ThingArn": (str, False),
        "Type": (str, True),
    }


class LoRaWANGateway(AWSProperty):
    props = {
        "GatewayEui": (str, True),
        "RfRegion": (str, True),
    }


class WirelessGateway(AWSObject):
    resource_type = "AWS::IoTWireless::WirelessGateway"

    props = {
        "Description": (str, False),
        "LastUplinkReceivedAt": (str, False),
        "LoRaWAN": (LoRaWANGateway, True),
        "Name": (str, False),
        "Tags": (Tags, False),
        "ThingArn": (str, False),
    }



from troposphere import GetAtt, Join, Ref, Template
from troposphere.awslambda import Code, Function, Permission
from troposphere.events import Rule, Target

t = Template()


# Create a Lambda function that will be mapped
code = [
    "var response = require('cfn-response');",
    "exports.handler = function(event, context) {",
    "   context.succeed('foobar!');",
    "   return 'foobar!';",
    "};",
]

# Create the Lambda function
foobar_function = t.add_resource(
    Function(
        "FoobarFunction",
        Code=Code(ZipFile=Join("", code)),
        Handler="index.handler",
        Role=GetAtt("LambdaExecutionRole", "Arn"),
        Runtime="nodejs",
    )
)

# Create the Event Target
foobar_target = Target(
    "FoobarTarget", Arn=GetAtt("FoobarFunction", "Arn"), Id="FooBarFunction1"
)

# Create the Event Rule
rule = t.add_resource(
    Rule(
        "FoobarRule",
        EventPattern={
            "source": ["aws.ec2"],
            "detail-type": ["EC2 Instance State-change Notification"],
            "detail": {"state": ["stopping"]},
        },
        Description="Foobar CloudWatch Event",
        State="ENABLED",
        Targets=[foobar_target],
    )
)

# Create Lambda Permission
permission = t.add_resource(
    Permission(
        "FoobarPermission",
        Action="lambda:invokeFunction",
        Principal="events.amazonaws.com",
        FunctionName=Ref(foobar_function),
        SourceArn=GetAtt(rule, "Arn"),
    )
)

print(t.to_json())



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class Assignment(AWSObject):
    resource_type = "AWS::SSO::Assignment"

    props = {
        "InstanceArn": (str, True),
        "PermissionSetArn": (str, True),
        "PrincipalId": (str, True),
        "PrincipalType": (str, True),
        "TargetId": (str, True),
        "TargetType": (str, True),
    }


class AccessControlAttributeValueSourceList(AWSProperty):
    props = {
        "AccessControlAttributeValueSourceList": ([str], False),
    }


class AccessControlAttributeValue(AWSProperty):
    props = {
        "Source": (AccessControlAttributeValueSourceList, True),
    }


class AccessControlAttribute(AWSProperty):
    props = {
        "Key": (str, True),
        "Value": (AccessControlAttributeValue, True),
    }


class InstanceAccessControlAttributeConfiguration(AWSObject):
    resource_type = "AWS::SSO::InstanceAccessControlAttributeConfiguration"

    props = {
        "AccessControlAttributes": ([AccessControlAttribute], False),
        "InstanceArn": (str, True),
    }


class PermissionSet(AWSObject):
    resource_type = "AWS::SSO::PermissionSet"

    props = {
        "Description": (str, False),
        "InlinePolicy": (dict, False),
        "InstanceArn": (str, True),
        "ManagedPolicies": ([str], False),
        "Name": (str, True),
        "RelayStateType": (str, False),
        "SessionDuration": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, Tags
from .validators import boolean, integer


class DBCluster(AWSObject):
    resource_type = "AWS::DocDB::DBCluster"

    props = {
        "AvailabilityZones": ([str], False),
        "BackupRetentionPeriod": (integer, False),
        "DBClusterIdentifier": (str, False),
        "DBClusterParameterGroupName": (str, False),
        "DBSubnetGroupName": (str, False),
        "DeletionProtection": (boolean, False),
        "EnableCloudwatchLogsExports": ([str], False),
        "EngineVersion": (str, False),
        "KmsKeyId": (str, False),
        "MasterUserPassword": (str, False),
        "MasterUsername": (str, False),
        "Port": (integer, False),
        "PreferredBackupWindow": (str, False),
        "PreferredMaintenanceWindow": (str, False),
        "SnapshotIdentifier": (str, False),
        "StorageEncrypted": (boolean, False),
        "Tags": (Tags, False),
        "VpcSecurityGroupIds": ([str], False),
    }


class DBClusterParameterGroup(AWSObject):
    resource_type = "AWS::DocDB::DBClusterParameterGroup"

    props = {
        "Description": (str, True),
        "Family": (str, True),
        "Name": (str, False),
        "Parameters": (dict, True),
        "Tags": (Tags, False),
    }


class DBInstance(AWSObject):
    resource_type = "AWS::DocDB::DBInstance"

    props = {
        "AutoMinorVersionUpgrade": (boolean, False),
        "AvailabilityZone": (str, False),
        "DBClusterIdentifier": (str, True),
        "DBInstanceClass": (str, True),
        "DBInstanceIdentifier": (str, False),
        "PreferredMaintenanceWindow": (str, False),
        "Tags": (Tags, False),
    }


class DBSubnetGroup(AWSObject):
    resource_type = "AWS::DocDB::DBSubnetGroup"

    props = {
        "DBSubnetGroupDescription": (str, True),
        "DBSubnetGroupName": (str, False),
        "SubnetIds": ([str], True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class Application(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::Application"

    props = {
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class Attributes(AWSProperty):
    props = {}


class AttributeGroup(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::AttributeGroup"

    props = {
        "Attributes": (Attributes, True),
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class AttributeGroupAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::AttributeGroupAssociation"

    props = {
        "Application": (str, True),
        "AttributeGroup": (str, True),
    }


class ResourceAssociation(AWSObject):
    resource_type = "AWS::ServiceCatalogAppRegistry::ResourceAssociation"

    props = {
        "Application": (str, True),
        "Resource": (str, True),
        "ResourceType": (str, True),
    }
